@@ -36,8 +36,7 @@ import (
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbac "k8s.io/api/rbac/v1"
-	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/types"
+	k8sruntime "k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/client-go/discovery"
 	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/builder"
@@ -54,16 +53,7 @@ import (
 	"k8s.io/apimachinery/pkg/api/errors"
 )
 
-var (
-	// CheServiceAccountName - service account name for che-server.
-	CheServiceAccountName = "che"
-)
-
 const (
-	failedValidationReason            = "InstallOrUpdateFailed"
-	failedNoOpenshiftUser             = "NoOpenshiftUsers"
-	failedNoIdentityProviders         = "NoIdentityProviders"
-	failedUnableToGetOAuth            = "UnableToGetOpenshiftOAuth"
 	warningNoIdentityProvidersMessage = "No Openshift identity providers."
 
 	AddIdentityProviderMessage      = "Openshift oAuth was disabled. How to add identity provider read in the Help Link:"
@@ -77,7 +67,7 @@ const (
 // CheClusterReconciler reconciles a CheCluster object
 type CheClusterReconciler struct {
 	Log    logr.Logger
-	Scheme *runtime.Scheme
+	Scheme *k8sruntime.Scheme
 
 	// This client, initialized using mgr.Client() above, is a split client
 	// that reads objects from the cache and writes to the apiserver
@@ -90,35 +80,41 @@ type CheClusterReconciler struct {
 	nonCachedClient client.Client
 	// A discovery client to check for the existence of certain APIs registered
 	// in the API Server
-	discoveryClient   discovery.DiscoveryInterface
-	tests             bool
-	userHandler       OpenShiftOAuthUserHandler
-	permissionChecker PermissionChecker
+	discoveryClient  discovery.DiscoveryInterface
+	tests            bool
+	userHandler      OpenShiftOAuthUserHandler
+	reconcileManager *deploy.ReconcileManager
 	// the namespace to which to limit the reconciliation. If empty, all namespaces are considered
 	namespace string
 }
 
 // NewReconciler returns a new CheClusterReconciler
-func NewReconciler(mgr ctrl.Manager, namespace string) (*CheClusterReconciler, error) {
-	noncachedClient, err := client.New(mgr.GetConfig(), client.Options{Scheme: mgr.GetScheme()})
-	if err != nil {
-		return nil, err
-	}
-	discoveryClient, err := discovery.NewDiscoveryClientForConfig(mgr.GetConfig())
-	if err != nil {
-		return nil, err
+func NewReconciler(
+	k8sclient client.Client,
+	noncachedClient client.Client,
+	discoveryClient discovery.DiscoveryInterface,
+	scheme *k8sruntime.Scheme,
+	namespace string) *CheClusterReconciler {
+
+	reconcileManager := deploy.NewReconcileManager()
+
+	// order does matter
+	if !util.IsTestMode() {
+		reconcileManager.RegisterReconciler(NewCheClusterValidator())
 	}
+	reconcileManager.RegisterReconciler(deploy.NewImagePuller())
+
 	return &CheClusterReconciler{
-		Scheme: mgr.GetScheme(),
+		Scheme: scheme,
 		Log:    ctrl.Log.WithName("controllers").WithName("CheCluster"),
 
-		client:            mgr.GetClient(),
-		nonCachedClient:   noncachedClient,
-		discoveryClient:   discoveryClient,
-		userHandler:       NewOpenShiftOAuthUserHandler(noncachedClient),
-		permissionChecker: &K8sApiPermissionChecker{},
-		namespace:         namespace,
-	}, nil
+		client:           k8sclient,
+		nonCachedClient:  noncachedClient,
+		discoveryClient:  discoveryClient,
+		userHandler:      NewOpenShiftOAuthUserHandler(noncachedClient),
+		namespace:        namespace,
+		reconcileManager: reconcileManager,
+	}
 }
 
 // SetupWithManager sets up the controller with the Manager.
@@ -140,16 +136,16 @@ func (r *CheClusterReconciler) SetupWithManager(mgr ctrl.Manager) error {
 		},
 	}
 
-	var toTrustedBundleConfigMapRequestMapper handler.ToRequestsFunc = func(obj handler.MapObject) []ctrl.Request {
-		isTrusted, reconcileRequest := isTrustedBundleConfigMap(mgr, obj)
+	var toTrustedBundleConfigMapRequestMapper handler.MapFunc = func(obj client.Object) []ctrl.Request {
+		isTrusted, reconcileRequest := IsTrustedBundleConfigMap(r.nonCachedClient, r.namespace, obj)
 		if isTrusted {
 			return []ctrl.Request{reconcileRequest}
 		}
 		return []ctrl.Request{}
 	}
 
-	var toEclipseCheRelatedObjRequestMapper handler.ToRequestsFunc = func(obj handler.MapObject) []ctrl.Request {
-		isEclipseCheRelatedObj, reconcileRequest := isEclipseCheRelatedObj(mgr, obj)
+	var toEclipseCheRelatedObjRequestMapper handler.MapFunc = func(obj client.Object) []ctrl.Request {
+		isEclipseCheRelatedObj, reconcileRequest := IsEclipseCheRelatedObj(r.nonCachedClient, r.namespace, obj)
 		if isEclipseCheRelatedObj {
 			return []ctrl.Request{reconcileRequest}
 		}
@@ -193,15 +189,15 @@ func (r *CheClusterReconciler) SetupWithManager(mgr ctrl.Manager) error {
 			OwnerType:    &orgv1.CheCluster{},
 		}).
 		Watches(&source.Kind{Type: &corev1.ConfigMap{}},
-			&handler.EnqueueRequestsFromMapFunc{ToRequests: toTrustedBundleConfigMapRequestMapper},
+			handler.EnqueueRequestsFromMapFunc(toTrustedBundleConfigMapRequestMapper),
 			builder.WithPredicates(onAllExceptGenericEventsPredicate),
 		).
 		Watches(&source.Kind{Type: &corev1.Secret{}},
-			&handler.EnqueueRequestsFromMapFunc{ToRequests: toEclipseCheRelatedObjRequestMapper},
+			handler.EnqueueRequestsFromMapFunc(toEclipseCheRelatedObjRequestMapper),
 			builder.WithPredicates(onAllExceptGenericEventsPredicate),
 		).
 		Watches(&source.Kind{Type: &corev1.ConfigMap{}},
-			&handler.EnqueueRequestsFromMapFunc{ToRequests: toEclipseCheRelatedObjRequestMapper},
+			handler.EnqueueRequestsFromMapFunc(toEclipseCheRelatedObjRequestMapper),
 			builder.WithPredicates(onAllExceptGenericEventsPredicate),
 		)
 
@@ -230,19 +226,20 @@ func (r *CheClusterReconciler) SetupWithManager(mgr ctrl.Manager) error {
 // move the current state of the cluster closer to the desired state.
 //
 // For more details, check Reconcile and its Result here:
-// - https://pkg.go.dev/sigs.k8s.io/controller-runtime@v0.6.3/pkg/reconcile
-func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error) {
+// - https://pkg.go.dev/sigs.k8s.io/controller-runtime@v0.9.5/pkg/reconcile
+func (r *CheClusterReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.Result, error) {
 	_ = r.Log.WithValues("checluster", req.NamespacedName)
 
+	tests := r.tests
 	clusterAPI := deploy.ClusterAPI{
 		Client:          r.client,
 		NonCachedClient: r.nonCachedClient,
 		DiscoveryClient: r.discoveryClient,
 		Scheme:          r.Scheme,
 	}
+
 	// Fetch the CheCluster instance
-	tests := r.tests
-	instance, err := r.GetCR(req)
+	checluster, err := r.GetCR(req)
 
 	if err != nil {
 		if errors.IsNotFound(err) {
@@ -257,44 +254,59 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 
 	deployContext := &deploy.DeployContext{
 		ClusterAPI: clusterAPI,
-		CheCluster: instance,
+		CheCluster: checluster,
+	}
+
+	if isCheGoingToBeUpdated(checluster) {
+		// Current operator is newer than deployed Che
+		backupCR, err := getBackupCRForUpdate(deployContext)
+		if err != nil {
+			if errors.IsNotFound(err) {
+				// Create a backup before updating current installation
+				if err := requestBackup(deployContext); err != nil {
+					return ctrl.Result{}, err
+				}
+				// Backup request is successfully submitted
+				// Give some time for the backup
+				return ctrl.Result{RequeueAfter: time.Second * 15}, nil
+			}
+			return ctrl.Result{}, err
+		}
+		if backupCR.Status.State == orgv1.STATE_IN_PROGRESS || backupCR.Status.State == "" {
+			// Backup is still in progress
+			return ctrl.Result{RequeueAfter: time.Second * 5}, nil
+		}
+		// Backup is done or failed
+		// Proceed anyway
+	}
+
+	if deployContext.CheCluster.ObjectMeta.DeletionTimestamp.IsZero() {
+		result, done, err := r.reconcileManager.ReconcileAll(deployContext)
+		if !done {
+			return result, err
+			// TODO: uncomment when all items added to ReconcilerManager
+			// } else {
+			// 	logrus.Info("Successfully reconciled.")
+			// 	return ctrl.Result{}, nil
+		}
+	} else {
+		r.reconcileManager.FinalizeAll(deployContext)
 	}
 
 	// Reconcile finalizers before CR is deleted
+	// TODO remove in favor of r.reconcileManager.FinalizeAll(deployContext)
 	r.reconcileFinalizers(deployContext)
 
-	// Reconcile the imagePuller section of the CheCluster
-	imagePullerResult, err := deploy.ReconcileImagePuller(deployContext)
-	if err != nil {
-		return imagePullerResult, err
-	}
-	if imagePullerResult.Requeue || imagePullerResult.RequeueAfter > 0 {
-		return imagePullerResult, err
-	}
-
-	// Check Che CR correctness
-	if !util.IsTestMode() {
-		if err := ValidateCheCR(instance); err != nil {
-			// Che cannot be deployed with current configuration.
-			// Print error message in logs and wait until the configuration is changed.
-			logrus.Error(err)
-			if err := deploy.SetStatusDetails(deployContext, failedValidationReason, err.Error(), ""); err != nil {
-				return reconcile.Result{}, err
-			}
-			return reconcile.Result{}, nil
-		}
-	}
-
-	if util.IsOpenShift4 && util.IsDeleteOAuthInitialUser(instance) {
+	if util.IsOpenShift4 && util.IsDeleteOAuthInitialUser(checluster) {
 		if err := r.userHandler.DeleteOAuthInitialUser(deployContext); err != nil {
 			logrus.Errorf("Unable to delete initial OpenShift OAuth user from a cluster. Cause: %s", err.Error())
-			instance.Spec.Auth.InitialOpenShiftOAuthUser = nil
+			checluster.Spec.Auth.InitialOpenShiftOAuthUser = nil
 			err := deploy.UpdateCheCRSpec(deployContext, "initialOpenShiftOAuthUser", "nil")
 			return reconcile.Result{}, err
 		}
 
-		instance.Spec.Auth.OpenShiftoAuth = nil
-		instance.Spec.Auth.InitialOpenShiftOAuthUser = nil
+		checluster.Spec.Auth.OpenShiftoAuth = nil
+		checluster.Spec.Auth.InitialOpenShiftOAuthUser = nil
 		updateFields := map[string]string{
 			"openShiftoAuth":            "nil",
 			"initialOpenShiftOAuthUser": "nil",
@@ -308,30 +320,30 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	}
 
 	// Update status if OpenShift initial user is deleted (in the previous step)
-	if instance.Spec.Auth.InitialOpenShiftOAuthUser == nil && instance.Status.OpenShiftOAuthUserCredentialsSecret != "" {
+	if checluster.Spec.Auth.InitialOpenShiftOAuthUser == nil && checluster.Status.OpenShiftOAuthUserCredentialsSecret != "" {
 		secret := &corev1.Secret{}
 		exists, err := getOpenShiftOAuthUserCredentialsSecret(deployContext, secret)
 		if err != nil {
 			// We should `Requeue` since we deal with cluster scope objects
 			return ctrl.Result{RequeueAfter: time.Second}, err
 		} else if !exists {
-			instance.Status.OpenShiftOAuthUserCredentialsSecret = ""
+			checluster.Status.OpenShiftOAuthUserCredentialsSecret = ""
 			if err := deploy.UpdateCheCRStatus(deployContext, "openShiftOAuthUserCredentialsSecret", ""); err != nil {
 				return reconcile.Result{}, err
 			}
 		}
 	}
 
-	if util.IsOpenShift && instance.Spec.DevWorkspace.Enable && instance.Spec.Auth.NativeUserMode == nil {
+	if util.IsOpenShift && checluster.Spec.DevWorkspace.Enable && checluster.Spec.Auth.NativeUserMode == nil {
 		newNativeUserModeValue := util.NewBoolPointer(true)
-		instance.Spec.Auth.NativeUserMode = newNativeUserModeValue
+		checluster.Spec.Auth.NativeUserMode = newNativeUserModeValue
 		if err := deploy.UpdateCheCRSpec(deployContext, "nativeUserMode", strconv.FormatBool(*newNativeUserModeValue)); err != nil {
 			return reconcile.Result{Requeue: true, RequeueAfter: time.Second * 1}, err
 		}
 	}
 
-	if util.IsOpenShift && instance.Spec.Auth.OpenShiftoAuth == nil {
-		if reconcileResult, err := r.autoEnableOAuth(deployContext, req, util.IsOpenShift4); err != nil {
+	if util.IsOpenShift && checluster.Spec.Auth.OpenShiftoAuth == nil {
+		if reconcileResult, err := r.autoEnableOAuth(deployContext); err != nil {
 			return reconcileResult, err
 		}
 	}
@@ -347,7 +359,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	}
 
 	// Read proxy configuration
-	proxy, err := r.getProxyConfiguration(deployContext)
+	proxy, err := GetProxyConfiguration(deployContext)
 	if err != nil {
 		r.Log.Error(err, "Error on reading proxy configuration")
 		return ctrl.Result{}, err
@@ -358,7 +370,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	if proxy.TrustedCAMapName != "" {
 		provisioned, err := r.putOpenShiftCertsIntoConfigMap(deployContext)
 		if !provisioned {
-			configMapName := instance.Spec.Server.ServerTrustStoreConfigMapName
+			configMapName := checluster.Spec.Server.ServerTrustStoreConfigMapName
 			if err != nil {
 				r.Log.Error(err, "Error on provisioning", "config map", configMapName)
 			} else {
@@ -381,13 +393,13 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 			// To use Openshift v4 OAuth, the OAuth endpoints are served from a namespace
 			// and NOT from the Openshift API Master URL (as in v3)
 			// So we also need the self-signed certificate to access them (same as the Che server)
-			(util.IsOpenShift4 && util.IsOAuthEnabled(instance) && !instance.Spec.Server.TlsSupport) {
+			(util.IsOpenShift4 && util.IsOAuthEnabled(checluster) && !checluster.Spec.Server.TlsSupport) {
 			if err := deploy.CreateTLSSecretFromEndpoint(deployContext, "", deploy.CheTLSSelfSignedCertificateSecretName); err != nil {
 				return ctrl.Result{}, err
 			}
 		}
 
-		if util.IsOAuthEnabled(instance) {
+		if util.IsOAuthEnabled(checluster) {
 			// create a secret with OpenShift API crt to be added to keystore that RH SSO will consume
 			apiUrl, apiInternalUrl, err := util.GetOpenShiftAPIUrls()
 			if err != nil {
@@ -401,8 +413,8 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		}
 	} else {
 		// Handle Che TLS certificates on Kubernetes infrastructure
-		if instance.Spec.Server.TlsSupport {
-			if instance.Spec.K8s.TlsSecretName != "" {
+		if checluster.Spec.Server.TlsSupport {
+			if checluster.Spec.K8s.TlsSecretName != "" {
 				// Self-signed certificate should be created to secure Che ingresses
 				result, err := deploy.K8sHandleCheTLSSecrets(deployContext)
 				if result.Requeue || result.RequeueAfter > 0 {
@@ -434,17 +446,13 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		return ctrl.Result{}, err
 	}
 
-	if err := deploy.SetStatusDetails(deployContext, "", "", ""); err != nil {
-		return ctrl.Result{}, err
-	}
-
 	// Create service account "che" for che-server component.
 	// "che" is the one which token is used to create workspace objects.
 	// Notice: Also we have on more "che-workspace" SA used by plugins like exec, terminal, metrics with limited privileges.
-	done, err = deploy.SyncServiceAccountToCluster(deployContext, CheServiceAccountName)
+	done, err = deploy.SyncServiceAccountToCluster(deployContext, deploy.CheServiceAccountName)
 	if !done {
 		if err != nil {
-			r.Log.Error(err, "")
+			logrus.Error(err)
 		}
 		return ctrl.Result{RequeueAfter: time.Second}, err
 	}
@@ -466,12 +474,12 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		return ctrl.Result{RequeueAfter: time.Second}, err
 	}
 
-	if len(instance.Spec.Server.CheClusterRoles) > 0 {
-		cheClusterRoles := strings.Split(instance.Spec.Server.CheClusterRoles, ",")
+	if len(checluster.Spec.Server.CheClusterRoles) > 0 {
+		cheClusterRoles := strings.Split(checluster.Spec.Server.CheClusterRoles, ",")
 		for _, cheClusterRole := range cheClusterRoles {
 			cheClusterRole := strings.TrimSpace(cheClusterRole)
 			cheClusterRoleBindingName := cheClusterRole
-			done, err := deploy.SyncClusterRoleBindingAndAddFinalizerToCluster(deployContext, cheClusterRoleBindingName, CheServiceAccountName, cheClusterRole)
+			done, err := deploy.SyncClusterRoleBindingAndAddFinalizerToCluster(deployContext, cheClusterRoleBindingName, deploy.CheServiceAccountName, cheClusterRole)
 			if !tests {
 				if !done {
 					logrus.Infof("Waiting on cluster role binding '%s' to be created", cheClusterRoleBindingName)
@@ -486,7 +494,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 
 	// If the user specified an additional cluster role to use for the Che workspace, create a role binding for it
 	// Use a role binding instead of a cluster role binding to keep the additional access scoped to the workspace's namespace
-	workspaceClusterRole := instance.Spec.Server.CheWorkspaceClusterRole
+	workspaceClusterRole := checluster.Spec.Server.CheWorkspaceClusterRole
 	if workspaceClusterRole != "" {
 		done, err := deploy.SyncRoleBindingToCluster(deployContext, "che-workspace-custom", "view", workspaceClusterRole, "ClusterRole")
 		if !done {
@@ -497,8 +505,8 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		}
 	}
 
-	if err := r.GenerateAndSaveFields(deployContext, req); err != nil {
-		instance, _ = r.GetCR(req)
+	if err := r.GenerateAndSaveFields(deployContext); err != nil {
+		_ = deploy.ReloadCheClusterCR(deployContext)
 		return ctrl.Result{Requeue: true, RequeueAfter: time.Second * 1}, err
 	}
 
@@ -525,7 +533,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	}
 
 	// create and provision Keycloak related objects
-	if !instance.Spec.Auth.ExternalIdentityProvider {
+	if !checluster.Spec.Auth.ExternalIdentityProvider {
 		provisioned, err := identity_provider.SyncIdentityProviderToCluster(deployContext)
 		if !provisioned {
 			if err != nil {
@@ -534,9 +542,9 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 			return ctrl.Result{}, err
 		}
 	} else {
-		keycloakURL := instance.Spec.Auth.IdentityProviderURL
-		if instance.Status.KeycloakURL != keycloakURL {
-			instance.Status.KeycloakURL = keycloakURL
+		keycloakURL := checluster.Spec.Auth.IdentityProviderURL
+		if checluster.Status.KeycloakURL != keycloakURL {
+			checluster.Status.KeycloakURL = keycloakURL
 			if err := deploy.UpdateCheCRStatus(deployContext, "status: Keycloak URL", keycloakURL); err != nil {
 				return reconcile.Result{}, err
 			}
@@ -544,7 +552,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	}
 
 	devfileRegistry := devfileregistry.NewDevfileRegistry(deployContext)
-	if !instance.Spec.Server.ExternalDevfileRegistry {
+	if !checluster.Spec.Server.ExternalDevfileRegistry {
 		done, err := devfileRegistry.SyncAll()
 		if !done {
 			if err != nil {
@@ -554,7 +562,7 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		}
 	}
 
-	if !instance.Spec.Server.ExternalPluginRegistry {
+	if !checluster.Spec.Server.ExternalPluginRegistry {
 		pluginRegistry := pluginregistry.NewPluginRegistry(deployContext)
 		done, err := pluginRegistry.SyncAll()
 		if !done {
@@ -564,9 +572,9 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 			return ctrl.Result{}, err
 		}
 	} else {
-		if instance.Spec.Server.PluginRegistryUrl != instance.Status.PluginRegistryURL {
-			instance.Status.PluginRegistryURL = instance.Spec.Server.PluginRegistryUrl
-			if err := deploy.UpdateCheCRStatus(deployContext, "status: Plugin Registry URL", instance.Spec.Server.PluginRegistryUrl); err != nil {
+		if checluster.Spec.Server.PluginRegistryUrl != checluster.Status.PluginRegistryURL {
+			checluster.Status.PluginRegistryURL = checluster.Spec.Server.PluginRegistryUrl
+			if err := deploy.UpdateCheCRStatus(deployContext, "status: Plugin Registry URL", checluster.Spec.Server.PluginRegistryUrl); err != nil {
 				return reconcile.Result{}, err
 			}
 		}
@@ -612,86 +620,49 @@ func (r *CheClusterReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 		// ignore error
 		deploy.DeleteFinalizer(deployContext, deploy.OAuthFinalizerName)
 		for {
-			instance.Status.OpenShiftoAuthProvisioned = false
+			checluster.Status.OpenShiftoAuthProvisioned = false
 			if err := deploy.UpdateCheCRStatus(deployContext, "status: provisioned with OpenShift identity provider", "false"); err != nil &&
 				errors.IsConflict(err) {
-				instance, _ = r.GetCR(req)
+				_ = deploy.ReloadCheClusterCR(deployContext)
 				continue
 			}
 			break
 		}
 		for {
-			instance.Spec.Auth.OAuthSecret = ""
-			instance.Spec.Auth.OAuthClientName = ""
+			checluster.Spec.Auth.OAuthSecret = ""
+			checluster.Spec.Auth.OAuthClientName = ""
 			if err := deploy.UpdateCheCRStatus(deployContext, "clean oAuth secret name and client name", ""); err != nil &&
 				errors.IsConflict(err) {
-				instance, _ = r.GetCR(req)
+				_ = deploy.ReloadCheClusterCR(deployContext)
 				continue
 			}
 			break
 		}
 	}
 
+	logrus.Info("Successfully reconciled.")
 	return ctrl.Result{}, nil
 }
 
-// isTrustedBundleConfigMap detects whether given config map is the config map with additional CA certificates to be trusted by Che
-func isTrustedBundleConfigMap(mgr ctrl.Manager, obj handler.MapObject) (bool, ctrl.Request) {
-	checlusters := &orgv1.CheClusterList{}
-	if err := mgr.GetClient().List(context.TODO(), checlusters, &client.ListOptions{}); err != nil {
-		return false, ctrl.Request{}
-	}
-
-	if len(checlusters.Items) != 1 {
-		return false, ctrl.Request{}
-	}
-
-	// Check if config map is the config map from CR
-	if checlusters.Items[0].Spec.Server.ServerTrustStoreConfigMapName != obj.Meta.GetName() {
-		// No, it is not form CR
-		// Check for labels
-
-		// Check for part of Che label
-		if value, exists := obj.Meta.GetLabels()[deploy.KubernetesPartOfLabelKey]; !exists || value != deploy.CheEclipseOrg {
-			// Labels do not match
-			return false, ctrl.Request{}
-		}
-
-		// Check for CA bundle label
-		if value, exists := obj.Meta.GetLabels()[deploy.CheCACertsConfigMapLabelKey]; !exists || value != deploy.CheCACertsConfigMapLabelValue {
-			// Labels do not match
-			return false, ctrl.Request{}
-		}
-	}
-
-	return true, ctrl.Request{
-		NamespacedName: types.NamespacedName{
-			Namespace: checlusters.Items[0].Namespace,
-			Name:      checlusters.Items[0].Name,
-		},
-	}
-}
-
-func (r *CheClusterReconciler) autoEnableOAuth(deployContext *deploy.DeployContext, request ctrl.Request, isOpenShift4 bool) (reconcile.Result, error) {
-	var message, reason string
+func (r *CheClusterReconciler) autoEnableOAuth(deployContext *deploy.DeployContext) (reconcile.Result, error) {
 	oauth := false
 	cr := deployContext.CheCluster
-	if isOpenShift4 {
+	if util.IsOpenShift4 {
 		openshitOAuth, err := GetOpenshiftOAuth(deployContext.ClusterAPI.NonCachedClient)
 		if err != nil {
-			message = "Unable to get Openshift oAuth. Cause: " + err.Error()
-			logrus.Error(message)
-			reason = failedUnableToGetOAuth
+			logrus.Error("Unable to get Openshift oAuth. Cause: " + err.Error())
 		} else {
 			if len(openshitOAuth.Spec.IdentityProviders) > 0 {
 				oauth = true
+			} else if util.IsNativeUserModeEnabled(deployContext.CheCluster) {
+				// enable OpenShift OAuth without adding initial OpenShift OAuth user
+				// since kubeadmin is a valid user for native user mode
+				oauth = true
 			} else if util.IsInitialOpenShiftOAuthUserEnabled(cr) {
 				provisioned, err := r.userHandler.SyncOAuthInitialUser(openshitOAuth, deployContext)
 				if err != nil {
-					message = warningNoIdentityProvidersMessage + " Operator tried to create initial OpenShift OAuth user for HTPasswd identity provider, but failed. Cause: " + err.Error()
-					logrus.Error(message)
+					logrus.Error(warningNoIdentityProvidersMessage + " Operator tried to create initial OpenShift OAuth user for HTPasswd identity provider, but failed. Cause: " + err.Error())
 					logrus.Info("To enable OpenShift OAuth, please add identity provider first: " + howToAddIdentityProviderLinkOS4)
-					reason = failedNoIdentityProviders
 					// Don't try to create initial user any more, codeready-operator shouldn't hang on this step.
 					cr.Spec.Auth.InitialOpenShiftOAuthUser = nil
 					if err := deploy.UpdateCheCRStatus(deployContext, "initialOpenShiftOAuthUser", ""); err != nil {
@@ -716,15 +687,11 @@ func (r *CheClusterReconciler) autoEnableOAuth(deployContext *deploy.DeployConte
 		users := &userv1.UserList{}
 		listOptions := &client.ListOptions{}
 		if err := r.nonCachedClient.List(context.TODO(), users, listOptions); err != nil {
-			message = failedUnableToGetOpenshiftUsers + " Cause: " + err.Error()
-			logrus.Error(message)
-			reason = failedNoOpenshiftUser
+			logrus.Error(failedUnableToGetOpenshiftUsers + " Cause: " + err.Error())
 		} else {
 			oauth = len(users.Items) >= 1
 			if !oauth {
-				message = warningNoRealUsersMessage + " " + howToConfigureOAuthLinkOS3
-				logrus.Warn(message)
-				reason = failedNoOpenshiftUser
+				logrus.Warn(warningNoRealUsersMessage + " " + howToConfigureOAuthLinkOS3)
 			}
 		}
 	}
@@ -737,40 +704,9 @@ func (r *CheClusterReconciler) autoEnableOAuth(deployContext *deploy.DeployConte
 		}
 	}
 
-	if message != "" && reason != "" {
-		if err := deploy.SetStatusDetails(deployContext, message, reason, ""); err != nil {
-			return reconcile.Result{}, err
-		}
-	}
-
 	return reconcile.Result{}, nil
 }
 
-// isEclipseCheRelatedObj indicates if there is a object with
-// the label 'app.kubernetes.io/part-of=che.eclipse.org' in a che namespace
-func isEclipseCheRelatedObj(mgr ctrl.Manager, obj handler.MapObject) (bool, ctrl.Request) {
-	checlusters := &orgv1.CheClusterList{}
-	if err := mgr.GetClient().List(context.TODO(), checlusters, &client.ListOptions{}); err != nil {
-		return false, ctrl.Request{}
-	}
-
-	if len(checlusters.Items) != 1 {
-		return false, ctrl.Request{}
-	}
-
-	if value, exists := obj.Meta.GetLabels()[deploy.KubernetesPartOfLabelKey]; !exists || value != deploy.CheEclipseOrg {
-		// Labels do not match
-		return false, ctrl.Request{}
-	}
-
-	return true, ctrl.Request{
-		NamespacedName: types.NamespacedName{
-			Namespace: checlusters.Items[0].Namespace,
-			Name:      checlusters.Items[0].Name,
-		},
-	}
-}
-
 func (r *CheClusterReconciler) reconcileFinalizers(deployContext *deploy.DeployContext) {
 	if util.IsOpenShift && util.IsOAuthEnabled(deployContext.CheCluster) {
 		if err := deploy.ReconcileOAuthClientFinalizer(deployContext); err != nil {
@@ -815,7 +751,7 @@ func (r *CheClusterReconciler) reconcileFinalizers(deployContext *deploy.DeployC
 			}
 
 			// Removes any legacy CRB https://github.com/eclipse/che/issues/19506
-			cheClusterRoleBindingName = deploy.GetLegacyUniqueClusterRoleBindingName(deployContext, CheServiceAccountName, cheClusterRole)
+			cheClusterRoleBindingName = deploy.GetLegacyUniqueClusterRoleBindingName(deployContext, deploy.CheServiceAccountName, cheClusterRole)
 			if err := deploy.ReconcileLegacyClusterRoleBindingFinalizer(deployContext, cheClusterRoleBindingName); err != nil {
 				logrus.Error(err)
 			}
