@@ -17,20 +17,17 @@ import (
 	"io/ioutil"
 	"os"
 	"strconv"
-	"strings"
-	"unicode/utf8"
 
 	mocks "github.com/eclipse-che/che-operator/mocks"
 
 	"reflect"
 	"time"
 
-	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
+	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/api/v1alpha1"
 	"github.com/golang/mock/gomock"
 
 	identity_provider "github.com/eclipse-che/che-operator/pkg/deploy/identity-provider"
 	"github.com/google/go-cmp/cmp"
-	"github.com/google/go-cmp/cmp/cmpopts"
 
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 	"github.com/eclipse-che/che-operator/pkg/util"
@@ -59,65 +56,20 @@ import (
 	fakeDiscovery "k8s.io/client-go/discovery/fake"
 	fakeclientset "k8s.io/client-go/kubernetes/fake"
 	"k8s.io/client-go/kubernetes/scheme"
-	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
 	configv1 "github.com/openshift/api/config/v1"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
-	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 	"sigs.k8s.io/yaml"
 
 	"testing"
 )
 
 var (
-	namespace       = "eclipse-che"
-	csvName         = "kubernetes-imagepuller-operator.v0.0.4"
-	packageManifest = &packagesv1.PackageManifest{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "kubernetes-imagepuller-operator",
-			Namespace: namespace,
-		},
-		Status: packagesv1.PackageManifestStatus{
-			CatalogSource:          "community-operators",
-			CatalogSourceNamespace: "olm",
-			DefaultChannel:         "stable",
-			PackageName:            "kubernetes-imagepuller-operator",
-		},
-	}
-	operatorGroup = &operatorsv1.OperatorGroup{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "kubernetes-imagepuller-operator",
-			Namespace: namespace,
-		},
-		Spec: operatorsv1.OperatorGroupSpec{
-			TargetNamespaces: []string{
-				namespace,
-			},
-		},
-	}
-	subscription = &operatorsv1alpha1.Subscription{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "kubernetes-imagepuller-operator",
-			Namespace: namespace,
-		},
-		Spec: &operatorsv1alpha1.SubscriptionSpec{
-			CatalogSource:          "community-operators",
-			Channel:                "stable",
-			CatalogSourceNamespace: "olm",
-			InstallPlanApproval:    operatorsv1alpha1.ApprovalAutomatic,
-			Package:                "kubernetes-imagepuller-operator",
-		},
-	}
-	valueTrue             = true
-	clusterServiceVersion = &operatorsv1alpha1.ClusterServiceVersion{
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace: namespace,
-			Name:      csvName,
-		},
-	}
+	namespace        = "eclipse-che"
 	nonEmptyUserList = &userv1.UserList{
 		Items: []userv1.User{
 			{
@@ -158,7 +110,6 @@ var (
 			Name: "cluster",
 		},
 	}
-	defaultImagePullerImages string
 )
 
 func init() {
@@ -170,8 +121,6 @@ func init() {
 			os.Setenv(env.Name, env.Value)
 		}
 	}
-	defaultImagePullerImages = "che-workspace-plugin-broker-metadata=" + os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_metadata") +
-		";che-workspace-plugin-broker-artifacts=" + os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_artifacts") + ";"
 }
 
 func TestNativeUserModeEnabled(t *testing.T) {
@@ -225,12 +174,13 @@ func TestNativeUserModeEnabled(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
 			scheme.AddKnownTypes(routev1.GroupVersion, route)
 			scheme.AddKnownTypes(oauth.SchemeGroupVersion, oAuthClient)
+			scheme.AddKnownTypes(configv1.SchemeGroupVersion, &configv1.Proxy{})
 			initCR := InitCheWithSimpleCR().DeepCopy()
 			testCase.initObjects = append(testCase.initObjects, initCR)
 
@@ -248,14 +198,9 @@ func TestNativeUserModeEnabled(t *testing.T) {
 				t.Fatal("Error creating fake discovery client")
 			}
 
-			r := &CheClusterReconciler{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				Scheme:          scheme,
-				tests:           true,
-				Log:             ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
+			r := NewReconciler(cli, nonCachedClient, fakeDiscovery, scheme, "")
+			r.tests = true
+
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      os.Getenv("CHE_FLAVOR"),
@@ -263,7 +208,7 @@ func TestNativeUserModeEnabled(t *testing.T) {
 				},
 			}
 
-			_, err := r.Reconcile(req)
+			_, err := r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
@@ -435,7 +380,7 @@ func TestCaseAutoDetectOAuth(t *testing.T) {
 	}
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
@@ -467,15 +412,10 @@ func TestCaseAutoDetectOAuth(t *testing.T) {
 				defer ctrl.Finish()
 			}
 
-			r := &CheClusterReconciler{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				Scheme:          scheme,
-				tests:           true,
-				userHandler:     userHandlerMock,
-				Log:             ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
+			r := NewReconciler(cli, nonCachedClient, fakeDiscovery, scheme, "")
+			r.userHandler = userHandlerMock
+			r.tests = true
+
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      os.Getenv("CHE_FLAVOR"),
@@ -486,7 +426,7 @@ func TestCaseAutoDetectOAuth(t *testing.T) {
 			util.IsOpenShift = true
 			util.IsOpenShift4 = !testCase.isOpenshift3
 
-			_, err := r.Reconcile(req)
+			_, err := r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
@@ -546,7 +486,7 @@ func TestEnsureServerExposureStrategy(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
@@ -565,14 +505,9 @@ func TestEnsureServerExposureStrategy(t *testing.T) {
 				t.Fatal("Error creating fake discovery client")
 			}
 
-			r := &CheClusterReconciler{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				Scheme:          scheme,
-				tests:           true,
-				Log:             ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
+			r := NewReconciler(cli, nonCachedClient, fakeDiscovery, scheme, "")
+			r.tests = true
+
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      os.Getenv("CHE_FLAVOR"),
@@ -580,7 +515,7 @@ func TestEnsureServerExposureStrategy(t *testing.T) {
 				},
 			}
 
-			_, err := r.Reconcile(req)
+			_, err := r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
@@ -675,7 +610,7 @@ func TestShouldSetUpCorrectlyDevfileRegistryURL(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
@@ -689,14 +624,9 @@ func TestShouldSetUpCorrectlyDevfileRegistryURL(t *testing.T) {
 			}
 			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
 
-			r := &CheClusterReconciler{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				Scheme:          scheme,
-				tests:           true,
-				Log:             ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
+			r := NewReconciler(cli, nonCachedClient, fakeDiscovery, scheme, "")
+			r.tests = true
+
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      os.Getenv("CHE_FLAVOR"),
@@ -707,7 +637,7 @@ func TestShouldSetUpCorrectlyDevfileRegistryURL(t *testing.T) {
 			util.IsOpenShift = testCase.isOpenShift
 			util.IsOpenShift4 = testCase.isOpenShift4
 
-			_, err := r.Reconcile(req)
+			_, err := r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
@@ -724,363 +654,17 @@ func TestShouldSetUpCorrectlyDevfileRegistryURL(t *testing.T) {
 	}
 }
 
-func TestImagePullerConfiguration(t *testing.T) {
-	oldBrokerMetaDataImage := strings.Split(os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_metadata"), ":")[0] + ":old"
-	oldBrokerArtifactsImage := strings.Split(os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_artifacts"), ":")[0] + ":old"
-	type testCase struct {
-		name                  string
-		initCR                *orgv1.CheCluster
-		initObjects           []runtime.Object
-		expectedCR            *orgv1.CheCluster
-		expectedOperatorGroup *operatorsv1.OperatorGroup
-		expectedSubscription  *operatorsv1alpha1.Subscription
-		expectedImagePuller   *chev1alpha1.KubernetesImagePuller
-		shouldDelete          bool
-	}
-
-	testCases := []testCase{
-		{
-			name:   "image puller enabled, no operatorgroup, should create an operatorgroup",
-			initCR: InitCheCRWithImagePullerEnabled(),
-			initObjects: []runtime.Object{
-				packageManifest,
-			},
-			expectedOperatorGroup: operatorGroup,
-		},
-		{
-			name:   "image puller enabled, operatorgroup exists, should create a subscription",
-			initCR: InitCheCRWithImagePullerEnabled(),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-			},
-			expectedSubscription: subscription,
-		},
-		{
-			name:       "image puller enabled, subscription created, should add finalizer",
-			initCR:     InitCheCRWithImagePullerEnabled(),
-			expectedCR: ExpectedCheCRWithImagePullerFinalizer(),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-			},
-		},
-		{
-			name:   "image puller enabled with finalizer but default values are empty, subscription exists, should update the CR",
-			initCR: InitCheCRWithImagePullerFinalizer(),
-			expectedCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:            os.Getenv("CHE_FLAVOR"),
-					Namespace:       namespace,
-					ResourceVersion: "1",
-					Finalizers: []string{
-						"kubernetesimagepullers.finalizers.che.eclipse.org",
-					},
-				},
-				Spec: orgv1.CheClusterSpec{
-					ImagePuller: orgv1.CheClusterSpecImagePuller{
-						Enable: true,
-						Spec: chev1alpha1.KubernetesImagePullerSpec{
-							DeploymentName: "kubernetes-image-puller",
-							ConfigMapName:  "k8s-image-puller",
-						},
-					},
-					Server: orgv1.CheClusterSpecServer{
-						ServerExposureStrategy: "multi-host",
-					},
-				},
-			},
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-			},
-		},
-		{
-			name:   "image puller enabled default values already set, subscription exists, should create a KubernetesImagePuller",
-			initCR: InitCheCRWithImagePullerEnabledAndDefaultValuesSet(),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: defaultImagePullerImages, ObjectMetaResourceVersion: "1"}),
-		},
-		{
-			name:   "image puller enabled, user images set, subscription exists, should create a KubernetesImagePuller with user images",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet("image=image_url"),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: "image=image_url", ObjectMetaResourceVersion: "1"}),
-		},
-		{
-			name:   "image puller enabled, one default image set, subscription exists, should update KubernetesImagePuller default image",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet("che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";"),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				InitImagePuller(ImagePullerOptions{SpecImages: "che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";", ObjectMetaResourceVersion: "1"}),
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: "che-workspace-plugin-broker-metadata=" + os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_metadata") + ";", ObjectMetaResourceVersion: "2"}),
-		},
-		{
-			name:   "image puller enabled, one default image set, subscription exists, should update KubernetesImagePuller default images while keeping user image",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet("image=image_url;che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";"),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				InitImagePuller(ImagePullerOptions{SpecImages: "image=image_url;che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";", ObjectMetaResourceVersion: "1"}),
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: "image=image_url;che-workspace-plugin-broker-metadata=" + os.Getenv("RELATED_IMAGE_che_workspace_plugin_broker_metadata") + ";", ObjectMetaResourceVersion: "2"}),
-		},
-		{
-			name:   "image puller enabled, default images set, subscription exists, should update KubernetesImagePuller default images",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet("che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";che-workspace-plugin-broker-artifacts=" + oldBrokerArtifactsImage + ";"),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				InitImagePuller(ImagePullerOptions{SpecImages: "che-workspace-plugin-broker-metadata=" + oldBrokerMetaDataImage + ";che-workspace-plugin-broker-artifacts=" + oldBrokerArtifactsImage + ";", ObjectMetaResourceVersion: "1"}),
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: defaultImagePullerImages, ObjectMetaResourceVersion: "2"}),
-		},
-		{
-			name:   "image puller enabled, latest default images set, subscription exists, should not update KubernetesImagePuller default images",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet(defaultImagePullerImages),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				InitImagePuller(ImagePullerOptions{SpecImages: defaultImagePullerImages, ObjectMetaResourceVersion: "1"}),
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: defaultImagePullerImages, ObjectMetaResourceVersion: "1"}),
-		},
-		{
-			name:   "image puller enabled, default images not set, subscription exists, should not set KubernetesImagePuller default images",
-			initCR: InitCheCRWithImagePullerEnabledAndImagesSet("image=image_url;"),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				InitImagePuller(ImagePullerOptions{SpecImages: "image=image_url;", ObjectMetaResourceVersion: "1"}),
-			},
-			expectedImagePuller: InitImagePuller(ImagePullerOptions{SpecImages: "image=image_url;", ObjectMetaResourceVersion: "1"}),
-		},
-		{
-			name:   "image puller enabled, KubernetesImagePuller created and spec in CheCluster is different, should update the KubernetesImagePuller",
-			initCR: InitCheCRWithImagePullerEnabledAndNewValuesSet(),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				getDefaultImagePuller(),
-			},
-			expectedImagePuller: &chev1alpha1.KubernetesImagePuller{
-				TypeMeta: metav1.TypeMeta{Kind: "KubernetesImagePuller", APIVersion: "che.eclipse.org/v1alpha1"},
-				ObjectMeta: metav1.ObjectMeta{
-					ResourceVersion: "2",
-					Name:            os.Getenv("CHE_FLAVOR") + "-image-puller",
-					Namespace:       namespace,
-					Labels: map[string]string{
-						"app":                       "che",
-						"component":                 "kubernetes-image-puller",
-						"app.kubernetes.io/part-of": os.Getenv("CHE_FLAVOR"),
-					},
-					OwnerReferences: []metav1.OwnerReference{
-						{
-							APIVersion:         "org.eclipse.che/v1",
-							Kind:               "CheCluster",
-							BlockOwnerDeletion: &valueTrue,
-							Controller:         &valueTrue,
-							Name:               os.Getenv("CHE_FLAVOR"),
-						},
-					},
-				},
-				Spec: chev1alpha1.KubernetesImagePullerSpec{
-					ConfigMapName:  "k8s-image-puller-trigger-update",
-					DeploymentName: "kubernetes-image-puller-trigger-update",
-				},
-			},
-		},
-		{
-			name:   "image puller already created, imagePuller disabled, should delete everything",
-			initCR: InitCheCRWithImagePullerDisabled(),
-			initObjects: []runtime.Object{
-				packageManifest,
-				operatorGroup,
-				subscription,
-				clusterServiceVersion,
-				getDefaultImagePuller(),
-			},
-			shouldDelete: true,
-		},
-	}
-
-	for _, testCase := range testCases {
-		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
-			orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
-			packagesv1.AddToScheme(scheme.Scheme)
-			operatorsv1alpha1.AddToScheme(scheme.Scheme)
-			operatorsv1.AddToScheme(scheme.Scheme)
-			chev1alpha1.AddToScheme(scheme.Scheme)
-			routev1.AddToScheme(scheme.Scheme)
-			testCase.initObjects = append(testCase.initObjects, testCase.initCR)
-			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
-			nonCachedClient := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
-			clientSet := fakeclientset.NewSimpleClientset()
-			fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
-			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{
-				{
-					GroupVersion: "packages.operators.coreos.com/v1",
-					APIResources: []metav1.APIResource{
-						{
-							Kind: "PackageManifest",
-						},
-					},
-				},
-				{
-					GroupVersion: "operators.coreos.com/v1alpha1",
-					APIResources: []metav1.APIResource{
-						{Kind: "OperatorGroup"},
-						{Kind: "Subscription"},
-						{Kind: "ClusterServiceVersion"},
-					},
-				},
-				{
-					GroupVersion: "che.eclipse.org/v1alpha1",
-					APIResources: []metav1.APIResource{
-						{Kind: "KubernetesImagePuller"},
-					},
-				},
-			}
-
-			if !ok {
-				t.Error("Error creating fake discovery client")
-				os.Exit(1)
-			}
-
-			r := &CheClusterReconciler{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				Scheme:          scheme.Scheme,
-				tests:           true,
-				Log:             ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
-			req := reconcile.Request{
-				NamespacedName: types.NamespacedName{
-					Name:      os.Getenv("CHE_FLAVOR"),
-					Namespace: namespace,
-				},
-			}
-			_, err := r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-
-			if testCase.expectedOperatorGroup != nil {
-				gotOperatorGroup := &operatorsv1.OperatorGroup{}
-				err := r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: testCase.expectedOperatorGroup.Namespace, Name: testCase.expectedOperatorGroup.Name}, gotOperatorGroup)
-				if err != nil {
-					t.Errorf("Error getting OperatorGroup: %v", err)
-				}
-				if !reflect.DeepEqual(testCase.expectedOperatorGroup.Spec.TargetNamespaces, gotOperatorGroup.Spec.TargetNamespaces) {
-					t.Errorf("Error expected target namespace %v but got %v", testCase.expectedOperatorGroup.Spec.TargetNamespaces, gotOperatorGroup.Spec.TargetNamespaces)
-				}
-			}
-			if testCase.expectedSubscription != nil {
-				gotSubscription := &operatorsv1alpha1.Subscription{}
-				err := r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: testCase.expectedSubscription.Namespace, Name: testCase.expectedSubscription.Name}, gotSubscription)
-				if err != nil {
-					t.Errorf("Error getting Subscription: %v", err)
-				}
-				if !reflect.DeepEqual(testCase.expectedSubscription.Spec, gotSubscription.Spec) {
-					t.Errorf("Error, subscriptions differ (-want +got) %v", cmp.Diff(testCase.expectedSubscription.Spec, gotSubscription.Spec))
-				}
-			}
-			// if expectedCR is not set, don't check it
-			if testCase.expectedCR != nil && !reflect.DeepEqual(testCase.initCR, testCase.expectedCR) {
-				gotCR := &orgv1.CheCluster{}
-				err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: namespace, Name: os.Getenv("CHE_FLAVOR")}, gotCR)
-				if err != nil {
-					t.Errorf("Error getting CheCluster: %v", err)
-				}
-				if !reflect.DeepEqual(testCase.expectedCR, gotCR) {
-					t.Errorf("Expected CR and CR returned from API server are different (-want +got): %v", cmp.Diff(testCase.expectedCR, gotCR))
-				}
-			}
-			if testCase.expectedImagePuller != nil {
-				gotImagePuller := &chev1alpha1.KubernetesImagePuller{}
-				err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: testCase.expectedImagePuller.Namespace, Name: testCase.expectedImagePuller.Name}, gotImagePuller)
-				if err != nil {
-					t.Errorf("Error getting KubernetesImagePuller: %v", err)
-				}
-
-				diff := cmp.Diff(testCase.expectedImagePuller, gotImagePuller, cmpopts.IgnoreFields(chev1alpha1.KubernetesImagePullerSpec{}, "Images"))
-				if diff != "" {
-					t.Errorf("Expected KubernetesImagePuller and KubernetesImagePuller returned from API server differ (-want, +got): %v", diff)
-				}
-
-				expectedImages := nonEmptySplit(testCase.expectedImagePuller.Spec.Images, ";")
-				if len(nonEmptySplit(testCase.expectedImagePuller.Spec.Images, ";")) != len(expectedImages) {
-					t.Errorf("Expected KubernetesImagePuller returns %d images", len(expectedImages))
-				}
-
-				for _, expectedImage := range expectedImages {
-					if !strings.Contains(gotImagePuller.Spec.Images, expectedImage) {
-						t.Errorf("Expected KubernetesImagePuller returned image: %s, but it did not", expectedImage)
-					}
-				}
-			}
-			if testCase.shouldDelete {
-
-				imagePuller := &chev1alpha1.KubernetesImagePuller{}
-				err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: namespace, Name: os.Getenv("CHE_FLAVOR") + "-image-puller"}, imagePuller)
-				if err == nil || !errors.IsNotFound(err) {
-					t.Fatalf("Should not have found KubernetesImagePuller: %v", err)
-				}
-
-				clusterServiceVersion := &operatorsv1alpha1.ClusterServiceVersion{}
-				err = r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: namespace, Name: csvName}, clusterServiceVersion)
-				if err == nil || !errors.IsNotFound(err) {
-					t.Fatalf("Should not have found ClusterServiceVersion: %v", err)
-				}
-
-				subscription := &operatorsv1alpha1.Subscription{}
-				err = r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: namespace, Name: "kubernetes-imagepuller-operator"}, subscription)
-				if err == nil || !errors.IsNotFound(err) {
-					t.Fatalf("Should not have found subscription: %v", err)
-				}
-
-				operatorGroup := &operatorsv1.OperatorGroup{}
-				err = r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: namespace, Name: "kubernetes-imagepuller-operator"}, operatorGroup)
-				if err == nil || !errors.IsNotFound(err) {
-					t.Fatalf("Should not have found subscription: %v", err)
-				}
-			}
-		})
-	}
-}
-
 func TestCheController(t *testing.T) {
+	var err error
+
 	util.IsOpenShift = true
 	util.IsOpenShift4 = false
 
 	cl, dc, scheme := Init()
 
 	// Create a ReconcileChe object with the scheme and fake client
-	r := &CheClusterReconciler{client: cl, nonCachedClient: cl, Scheme: &scheme, discoveryClient: dc, tests: true, Log: ctrl.Log.WithName("controllers").WithName("CheCluster")}
+	r := NewReconciler(cl, cl, dc, &scheme, "")
+	r.tests = true
 
 	// get CR
 	cheCR := &orgv1.CheCluster{
@@ -1103,21 +687,12 @@ func TestCheController(t *testing.T) {
 		},
 	}
 
-	_, err := r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
-	}
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
-	}
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
-	}
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
+	reconcileLoops := 4
+	for i := 0; i < reconcileLoops; i++ {
+		_, err = r.Reconcile(context.TODO(), req)
+		if err != nil {
+			t.Fatalf("reconcile: (%v)", err)
+		}
 	}
 
 	// get devfile-registry configmap
@@ -1126,6 +701,12 @@ func TestCheController(t *testing.T) {
 		t.Errorf("ConfigMap %s not found: %s", devfilecm.Name, err)
 	}
 
+	// get plugin-registry configmap
+	pluginRegistrycm := &corev1.ConfigMap{}
+	if err := cl.Get(context.TODO(), types.NamespacedName{Name: deploy.DevfileRegistryName, Namespace: cheCR.Namespace}, pluginRegistrycm); err != nil {
+		t.Errorf("ConfigMap %s not found: %s", pluginRegistrycm.Name, err)
+	}
+
 	// get CR
 	if err := cl.Get(context.TODO(), types.NamespacedName{Name: os.Getenv("CHE_FLAVOR"), Namespace: namespace}, cheCR); err != nil {
 		t.Errorf("CR not found")
@@ -1138,17 +719,11 @@ func TestCheController(t *testing.T) {
 	}
 
 	// reconcile again
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
-	}
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
-	}
-	_, err = r.Reconcile(req)
-	if err != nil {
-		t.Fatalf("reconcile: (%v)", err)
+	for i := 0; i < reconcileLoops; i++ {
+		_, err = r.Reconcile(context.TODO(), req)
+		if err != nil {
+			t.Fatalf("reconcile: (%v)", err)
+		}
 	}
 
 	// get configmap
@@ -1173,7 +748,9 @@ func TestCheController(t *testing.T) {
 
 	// run a few checks to make sure the operator reconciled tls routes and updated configmap
 	if cm.Data["CHE_INFRA_OPENSHIFT_TLS__ENABLED"] != "true" {
-		t.Errorf("ConfigMap wasn't updated. Extecting true, got: %s", cm.Data["CHE_INFRA_OPENSHIFT_TLS__ENABLED"])
+		// If the test fails here without obvious reason, it could mean that there was not enought reconcile loops before.
+		// To fix the above problem, just increase reconcileLoops variable above.
+		t.Errorf("ConfigMap wasn't updated. Expecting true, but got: %s", cm.Data["CHE_INFRA_OPENSHIFT_TLS__ENABLED"])
 	}
 	route := &routev1.Route{}
 	if err := cl.Get(context.TODO(), types.NamespacedName{Name: deploy.DefaultCheFlavor(cheCR), Namespace: cheCR.Namespace}, route); err != nil {
@@ -1194,7 +771,7 @@ func TestCheController(t *testing.T) {
 		t.Error("Failed to update CheCluster custom resource")
 	}
 
-	_, err = r.Reconcile(req)
+	_, err = r.Reconcile(context.TODO(), req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
@@ -1253,7 +830,7 @@ func TestCheController(t *testing.T) {
 	postgresDeployment := &appsv1.Deployment{}
 	err = r.client.Get(context.TODO(), types.NamespacedName{Name: deploy.PostgresName, Namespace: cheCR.Namespace}, postgresDeployment)
 	err = r.client.Delete(context.TODO(), postgresDeployment)
-	_, err = r.Reconcile(req)
+	_, err = r.Reconcile(context.TODO(), req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
@@ -1276,7 +853,7 @@ func TestCheController(t *testing.T) {
 	if err = r.client.Delete(context.TODO(), pvc); err != nil {
 		t.Fatalf("Failed to delete PVC %s: %s", pvc.Name, err)
 	}
-	_, err = r.Reconcile(req)
+	_, err = r.Reconcile(context.TODO(), req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
@@ -1321,12 +898,13 @@ func TestCheController(t *testing.T) {
 func TestConfiguringLabelsForRoutes(t *testing.T) {
 	util.IsOpenShift = true
 	// Set the logger to development mode for verbose logs.
-	logf.SetLogger(logf.ZapLogger(true))
+	logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 	cl, dc, scheme := Init()
 
 	// Create a ReconcileChe object with the scheme and fake client
-	r := &CheClusterReconciler{client: cl, nonCachedClient: cl, Scheme: &scheme, discoveryClient: dc, tests: true, Log: ctrl.Log.WithName("controllers").WithName("CheCluster")}
+	r := NewReconciler(cl, cl, dc, &scheme, "")
+	r.tests = true
 
 	// get CR
 	cheCR := &orgv1.CheCluster{}
@@ -1344,7 +922,7 @@ func TestConfiguringLabelsForRoutes(t *testing.T) {
 	}
 
 	// reconcile
-	_, err := r.Reconcile(req)
+	_, err := r.Reconcile(context.TODO(), req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
@@ -1359,7 +937,7 @@ func TestConfiguringLabelsForRoutes(t *testing.T) {
 	}
 
 	// reconcile again
-	_, err = r.Reconcile(req)
+	_, err = r.Reconcile(context.TODO(), req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
@@ -1430,7 +1008,7 @@ func TestShouldDelegatePermissionsForCheWorkspaces(t *testing.T) {
 	}
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
@@ -1462,15 +1040,9 @@ func TestShouldDelegatePermissionsForCheWorkspaces(t *testing.T) {
 				defer ctrl.Finish()
 			}
 
-			r := &CheClusterReconciler{
-				client:            cli,
-				nonCachedClient:   nonCachedClient,
-				discoveryClient:   fakeDiscovery,
-				Scheme:            scheme,
-				permissionChecker: m,
-				tests:             true,
-				Log:               ctrl.Log.WithName("controllers").WithName("CheCluster"),
-			}
+			r := NewReconciler(cli, nonCachedClient, fakeDiscovery, scheme, "")
+			r.tests = true
+
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      os.Getenv("CHE_FLAVOR"),
@@ -1478,11 +1050,11 @@ func TestShouldDelegatePermissionsForCheWorkspaces(t *testing.T) {
 				},
 			}
 
-			_, err := r.Reconcile(req)
+			_, err := r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
-			_, err = r.Reconcile(req)
+			_, err = r.Reconcile(context.TODO(), req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
@@ -1625,232 +1197,3 @@ func InitCheWithSimpleCR() *orgv1.CheCluster {
 		},
 	}
 }
-
-func InitCheCRWithImagePullerEnabled() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-			},
-			Server: orgv1.CheClusterSpecServer{
-				ServerExposureStrategy: "multi-host",
-			},
-		},
-	}
-}
-
-func InitCheCRWithImagePullerFinalizer() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-			Finalizers: []string{
-				"kubernetesimagepullers.finalizers.che.eclipse.org",
-			},
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-			},
-			Server: orgv1.CheClusterSpecServer{
-				ServerExposureStrategy: "multi-host",
-			},
-		},
-	}
-}
-
-func ExpectedCheCRWithImagePullerFinalizer() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		TypeMeta: metav1.TypeMeta{
-			Kind:       "CheCluster",
-			APIVersion: "org.eclipse.che/v1",
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-			Finalizers: []string{
-				"kubernetesimagepullers.finalizers.che.eclipse.org",
-			},
-			ResourceVersion: "1",
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-			},
-			Server: orgv1.CheClusterSpecServer{
-				ServerExposureStrategy: "multi-host",
-			},
-		},
-	}
-}
-
-func InitCheCRWithImagePullerDisabled() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: false,
-			},
-		},
-	}
-}
-
-func InitCheCRWithImagePullerEnabledAndDefaultValuesSet() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-			Finalizers: []string{
-				"kubernetesimagepullers.finalizers.che.eclipse.org",
-			},
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-				Spec: chev1alpha1.KubernetesImagePullerSpec{
-					DeploymentName: "kubernetes-image-puller",
-					ConfigMapName:  "k8s-image-puller",
-				},
-			},
-			Auth: orgv1.CheClusterSpecAuth{
-				OpenShiftoAuth: util.NewBoolPointer(false),
-			},
-		},
-	}
-}
-
-func InitCheCRWithImagePullerEnabledAndImagesSet(images string) *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-			Finalizers: []string{
-				"kubernetesimagepullers.finalizers.che.eclipse.org",
-			},
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-				Spec: chev1alpha1.KubernetesImagePullerSpec{
-					DeploymentName: "kubernetes-image-puller",
-					ConfigMapName:  "k8s-image-puller",
-					Images:         images,
-				},
-			},
-			Auth: orgv1.CheClusterSpecAuth{
-				OpenShiftoAuth: util.NewBoolPointer(false),
-			},
-		},
-	}
-}
-
-func InitCheCRWithImagePullerEnabledAndNewValuesSet() *orgv1.CheCluster {
-	return &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR"),
-			Namespace: namespace,
-			Finalizers: []string{
-				"kubernetesimagepullers.finalizers.che.eclipse.org",
-			},
-		},
-		Spec: orgv1.CheClusterSpec{
-			ImagePuller: orgv1.CheClusterSpecImagePuller{
-				Enable: true,
-				Spec: chev1alpha1.KubernetesImagePullerSpec{
-					DeploymentName: "kubernetes-image-puller-trigger-update",
-					ConfigMapName:  "k8s-image-puller-trigger-update",
-				},
-			},
-			Auth: orgv1.CheClusterSpecAuth{
-				OpenShiftoAuth: util.NewBoolPointer(false),
-			},
-		},
-	}
-}
-
-type ImagePullerOptions struct {
-	SpecImages                string
-	ObjectMetaResourceVersion string
-}
-
-func InitImagePuller(options ImagePullerOptions) *chev1alpha1.KubernetesImagePuller {
-	return &chev1alpha1.KubernetesImagePuller{
-		TypeMeta: metav1.TypeMeta{
-			APIVersion: "che.eclipse.org/v1alpha1",
-			Kind:       "KubernetesImagePuller",
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR") + "-image-puller",
-			Namespace: namespace,
-			Labels: map[string]string{
-				"app.kubernetes.io/part-of": os.Getenv("CHE_FLAVOR"),
-				"app":                       "che",
-				"component":                 "kubernetes-image-puller",
-			},
-			ResourceVersion: options.ObjectMetaResourceVersion,
-			OwnerReferences: []metav1.OwnerReference{
-				{
-					APIVersion:         "org.eclipse.che/v1",
-					Kind:               "CheCluster",
-					Controller:         &valueTrue,
-					BlockOwnerDeletion: &valueTrue,
-					Name:               os.Getenv("CHE_FLAVOR"),
-				},
-			},
-		},
-		Spec: chev1alpha1.KubernetesImagePullerSpec{
-			DeploymentName: "kubernetes-image-puller",
-			ConfigMapName:  "k8s-image-puller",
-			Images:         options.SpecImages,
-		},
-	}
-}
-
-func getDefaultImagePuller() *chev1alpha1.KubernetesImagePuller {
-	return &chev1alpha1.KubernetesImagePuller{
-		TypeMeta: metav1.TypeMeta{
-			APIVersion: "che.eclipse.org/v1alpha1",
-			Kind:       "KubernetesImagePuller",
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      os.Getenv("CHE_FLAVOR") + "-image-puller",
-			Namespace: namespace,
-			Labels: map[string]string{
-				"app.kubernetes.io/part-of": os.Getenv("CHE_FLAVOR"),
-				"app":                       "che",
-				"component":                 "kubernetes-image-puller",
-			},
-			ResourceVersion: "1",
-			OwnerReferences: []metav1.OwnerReference{
-				{
-					APIVersion:         "org.eclipse.che/v1",
-					Kind:               "CheCluster",
-					Controller:         &valueTrue,
-					BlockOwnerDeletion: &valueTrue,
-					Name:               os.Getenv("CHE_FLAVOR"),
-				},
-			},
-		},
-		Spec: chev1alpha1.KubernetesImagePullerSpec{
-			DeploymentName: "kubernetes-image-puller",
-			ConfigMapName:  "k8s-image-puller",
-			Images:         defaultImagePullerImages,
-		},
-	}
-}
-
-// Split string by separator without empty elems
-func nonEmptySplit(lineToSplit string, separator string) []string {
-	splitFn := func(c rune) bool {
-		runeChar, _ := utf8.DecodeRuneInString(separator)
-		return c == runeChar
-	}
-	return strings.FieldsFunc(lineToSplit, splitFn)
-}
