@@ -86,7 +86,7 @@ func (r *ReconcileCheClusterRestore) SetupWithManager(mgr ctrl.Manager) error {
 // and what is in the CheClusterRestore.Spec
 // The Controller will requeue the Request to be processed again if the returned error is non-nil or
 // Result.Requeue is true, otherwise upon completion it will remove the work from the queue.
-func (r *ReconcileCheClusterRestore) Reconcile(request ctrl.Request) (ctrl.Result, error) {
+func (r *ReconcileCheClusterRestore) Reconcile(ctx context.Context, request ctrl.Request) (ctrl.Result, error) {
 	// Fetch the CheClusterRestore instance
 	restoreCR := &chev1.CheClusterRestore{}
 	err := r.client.Get(context.TODO(), request.NamespacedName, restoreCR)
@@ -148,7 +148,8 @@ func (r *ReconcileCheClusterRestore) doReconcile(restoreCR *chev1.CheClusterRest
 	if backupServerConfigName == "" {
 		// Try to find backup server configuration in the same namespace
 		cheBackupServersConfigurationList := &chev1.CheBackupServerConfigurationList{}
-		if err := r.client.List(context.TODO(), cheBackupServersConfigurationList); err != nil {
+		listOptions := &client.ListOptions{Namespace: restoreCR.GetNamespace()}
+		if err := r.client.List(context.TODO(), cheBackupServersConfigurationList, listOptions); err != nil {
 			return false, err
 		}
 		if len(cheBackupServersConfigurationList.Items) != 1 {
