@@ -6,6 +6,12 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/eclipse-che/che-operator/pkg/util"
+
+	"github.com/stretchr/testify/assert"
+
+	"github.com/eclipse-che/che-operator/pkg/deploy/gateway"
+
 	dw "github.com/devfile/api/v2/pkg/apis/workspaces/v1alpha2"
 	"github.com/devfile/api/v2/pkg/attributes"
 	dwo "github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
@@ -17,7 +23,6 @@ import (
 	"github.com/eclipse-che/che-operator/api/v2alpha1"
 	controller "github.com/eclipse-che/che-operator/controllers/devworkspace"
 	"github.com/eclipse-che/che-operator/controllers/devworkspace/defaults"
-	"github.com/eclipse-che/che-operator/pkg/util"
 	routev1 "github.com/openshift/api/route/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -70,7 +75,7 @@ func getSpecObjectsForManager(t *testing.T, mgr *v2alpha1.CheCluster, routing *d
 
 	// we need to do 1 round of che manager reconciliation so that the solver gets initialized
 	cheRecon := controller.New(cl, scheme)
-	_, err = cheRecon.Reconcile(reconcile.Request{NamespacedName: types.NamespacedName{Name: mgr.Name, Namespace: mgr.Namespace}})
+	_, err = cheRecon.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: mgr.Name, Namespace: mgr.Namespace}})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -81,7 +86,7 @@ func getSpecObjectsForManager(t *testing.T, mgr *v2alpha1.CheCluster, routing *d
 	}
 
 	// now we need a second round of che manager reconciliation so that it proclaims the che gateway as established
-	cheRecon.Reconcile(reconcile.Request{NamespacedName: types.NamespacedName{Name: "che", Namespace: "ns"}})
+	cheRecon.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "che", Namespace: "ns"}})
 
 	return cl, solver, objs
 }
@@ -160,6 +165,7 @@ func relocatableDevWorkspaceRouting() *dwo.DevWorkspaceRouting {
 						Path:       "/1/",
 						Attributes: attributes.Attributes{
 							urlRewriteSupportedEndpointAttributeName: apiext.JSON{Raw: []byte("\"true\"")},
+							string(dwo.TypeEndpointAttribute):        apiext.JSON{Raw: []byte("\"main\"")},
 						},
 					},
 					{
@@ -187,10 +193,9 @@ func relocatableDevWorkspaceRouting() *dwo.DevWorkspaceRouting {
 	}
 }
 
-func TestCreateRelocatedObjects(t *testing.T) {
+func TestCreateRelocatedObjectsK8S(t *testing.T) {
 	util.IsOpenShift = false
 	util.IsOpenShift4 = false
-
 	cl, _, objs := getSpecObjects(t, relocatableDevWorkspaceRouting())
 
 	t.Run("noIngresses", func(t *testing.T) {
@@ -205,9 +210,27 @@ func TestCreateRelocatedObjects(t *testing.T) {
 		}
 	})
 
-	t.Run("noPodAdditions", func(t *testing.T) {
-		if objs.PodAdditions != nil {
-			t.Error()
+	t.Run("testPodAdditions", func(t *testing.T) {
+		if len(objs.PodAdditions.Containers) != 1 || objs.PodAdditions.Containers[0].Name != wsGatewayName {
+			t.Error("expected Container pod addition with Workspace Gateway. Got ", objs.PodAdditions)
+		}
+		if len(objs.PodAdditions.Volumes) != 1 || objs.PodAdditions.Volumes[0].Name != wsGatewayName {
+			t.Error("expected Volume pod addition for workspace gateway. Got ", objs.PodAdditions)
+		}
+
+		if objs.PodAdditions.Containers[0].Resources.Requests.Memory() == nil {
+			t.Error("expected addition pod Container Memory request to be set")
+		}
+		if objs.PodAdditions.Containers[0].Resources.Requests.Cpu() == nil {
+			t.Error("expected addition po Container CPU request to be set")
+		}
+
+		if objs.PodAdditions.Containers[0].Resources.Limits.Memory() == nil {
+			t.Error("expected addition po Container Memory limit to be set")
+		}
+
+		if objs.PodAdditions.Containers[0].Resources.Limits.Cpu() == nil {
+			t.Error("expected addition po Container CPU limit to be set")
 		}
 	})
 
@@ -232,35 +255,43 @@ func TestCreateRelocatedObjects(t *testing.T) {
 		cms := &corev1.ConfigMapList{}
 		cl.List(context.TODO(), cms)
 
-		if len(cms.Items) != 1 {
-			t.Errorf("there should be 1 configmap created for the gateway config of the workspace but there were: %d", len(cms.Items))
+		if len(cms.Items) != 2 {
+			t.Errorf("there should be 2 configmaps created for the gateway config of the workspace but there were: %d", len(cms.Items))
 		}
 
+		var workspaceMainCfg *corev1.ConfigMap
 		var workspaceCfg *corev1.ConfigMap
-
 		for _, cfg := range cms.Items {
-			if cfg.Name == "wsid" {
-				workspaceCfg = &cfg
+			if cfg.Name == "wsid-route" && cfg.Namespace == "ns" {
+				workspaceMainCfg = cfg.DeepCopy()
+			}
+			if cfg.Name == "wsid-route" && cfg.Namespace == "ws" {
+				workspaceCfg = cfg.DeepCopy()
 			}
 		}
 
-		if workspaceCfg == nil {
+		if workspaceMainCfg == nil {
 			t.Fatalf("traefik configuration for the workspace not found")
 		}
 
-		traefikWorkspaceConfig := workspaceCfg.Data["wsid.yml"]
+		traefikMainWorkspaceConfig := workspaceMainCfg.Data["wsid.yml"]
 
+		if len(traefikMainWorkspaceConfig) == 0 {
+			t.Fatal("No traefik config file found in the main workspace config configmap")
+		}
+
+		traefikWorkspaceConfig := workspaceCfg.Data["workspace.yml"]
 		if len(traefikWorkspaceConfig) == 0 {
 			t.Fatal("No traefik config file found in the workspace config configmap")
 		}
 
-		workspaceConfig := traefikConfig{}
+		workspaceConfig := gateway.TraefikConfig{}
 		if err := yaml.Unmarshal([]byte(traefikWorkspaceConfig), &workspaceConfig); err != nil {
 			t.Fatal(err)
 		}
 
 		if len(workspaceConfig.HTTP.Routers) != 1 {
-			t.Fatalf("Expected exactly one traefik router but got %d", len(workspaceConfig.HTTP.Routers))
+			t.Fatalf("Expected 1 traefik router but got %d", len(workspaceConfig.HTTP.Routers))
 		}
 
 		wsid := "wsid-m1-9999"
@@ -268,15 +299,15 @@ func TestCreateRelocatedObjects(t *testing.T) {
 			t.Fatal("traefik config doesn't contain expected workspace configuration")
 		}
 
-		if len(workspaceConfig.HTTP.Routers[wsid].Middlewares) != 3 {
-			t.Fatalf("Expected 3 middlewares in router but got '%d'", len(workspaceConfig.HTTP.Routers[wsid].Middlewares))
+		if len(workspaceConfig.HTTP.Routers[wsid].Middlewares) != 1 {
+			t.Fatalf("Expected 1 middlewares in router but got '%d'", len(workspaceConfig.HTTP.Routers[wsid].Middlewares))
 		}
 
-		if len(workspaceConfig.HTTP.Middlewares) != 3 {
-			t.Fatalf("Expected 3 middlewares set but got '%d'", len(workspaceConfig.HTTP.Middlewares))
+		if len(workspaceConfig.HTTP.Middlewares) != 1 {
+			t.Fatalf("Expected 1 middlewares set but got '%d'", len(workspaceConfig.HTTP.Middlewares))
 		}
 
-		mwares := []string{wsid + "-auth", wsid + "-prefix", wsid + "-header"}
+		mwares := []string{wsid + gateway.StripPrefixMiddlewareSuffix}
 		for _, mware := range mwares {
 			if _, ok := workspaceConfig.HTTP.Middlewares[mware]; !ok {
 				t.Fatalf("traefik config doesn't set middleware '%s'", mware)
@@ -295,6 +326,172 @@ func TestCreateRelocatedObjects(t *testing.T) {
 	})
 }
 
+func TestCreateRelocatedObjectsOpenshift(t *testing.T) {
+	util.IsOpenShift = true
+	util.IsOpenShift4 = true
+
+	cl, _, objs := getSpecObjects(t, relocatableDevWorkspaceRouting())
+
+	assert.Empty(t, objs.Ingresses)
+	assert.Empty(t, objs.Routes)
+
+	t.Run("testPodAdditions", func(t *testing.T) {
+		assert.Len(t, objs.PodAdditions.Containers, 1)
+		assert.Equal(t, objs.PodAdditions.Containers[0].Name, wsGatewayName)
+
+		assert.Len(t, objs.PodAdditions.Volumes, 1)
+		assert.Equal(t, objs.PodAdditions.Volumes[0].Name, wsGatewayName)
+	})
+
+	t.Run("traefikConfig", func(t *testing.T) {
+		cms := &corev1.ConfigMapList{}
+		cl.List(context.TODO(), cms)
+
+		assert.Len(t, cms.Items, 2)
+
+		var workspaceMainCfg *corev1.ConfigMap
+		var workspaceCfg *corev1.ConfigMap
+		for _, cfg := range cms.Items {
+			if cfg.Name == "wsid-route" && cfg.Namespace == "ns" {
+				workspaceMainCfg = cfg.DeepCopy()
+			}
+			if cfg.Name == "wsid-route" && cfg.Namespace == "ws" {
+				workspaceCfg = cfg.DeepCopy()
+			}
+		}
+
+		assert.NotNil(t, workspaceMainCfg, "traefik configuration for the workspace not found")
+
+		traefikMainWorkspaceConfig := workspaceMainCfg.Data["wsid.yml"]
+		assert.NotEmpty(t, traefikMainWorkspaceConfig, "No traefik config file found in the main workspace config configmap")
+
+		traefikWorkspaceConfig := workspaceCfg.Data["workspace.yml"]
+		assert.NotEmpty(t, traefikWorkspaceConfig, "No traefik config file found in the workspace config configmap")
+
+		workspaceConfig := gateway.TraefikConfig{}
+		assert.NoError(t, yaml.Unmarshal([]byte(traefikWorkspaceConfig), &workspaceConfig))
+
+		wsid := "wsid-m1-9999"
+		assert.Contains(t, workspaceConfig.HTTP.Routers, wsid)
+		assert.Len(t, workspaceConfig.HTTP.Routers[wsid].Middlewares, 2)
+
+		workspaceMainConfig := gateway.TraefikConfig{}
+		assert.NoError(t, yaml.Unmarshal([]byte(traefikMainWorkspaceConfig), &workspaceMainConfig))
+		assert.Len(t, workspaceMainConfig.HTTP.Middlewares, 3)
+
+		wsid = "wsid"
+		mwares := []string{
+			wsid + gateway.AuthMiddlewareSuffix,
+			wsid + gateway.StripPrefixMiddlewareSuffix,
+			wsid + gateway.HeaderRewriteMiddlewareSuffix}
+		for _, mware := range mwares {
+			assert.Contains(t, workspaceMainConfig.HTTP.Middlewares, mware)
+
+			found := false
+			for _, r := range workspaceMainConfig.HTTP.Routers[wsid].Middlewares {
+				if r == mware {
+					found = true
+				}
+			}
+			assert.Truef(t, found, "traefik config route doesn't set middleware '%s'", mware)
+		}
+
+		t.Run("testHealthzEndpointInMainWorkspaceRoute", func(t *testing.T) {
+			healthzName := "9999-healthz"
+			assert.Contains(t, workspaceMainConfig.HTTP.Routers, healthzName)
+			assert.Equal(t, workspaceMainConfig.HTTP.Routers[healthzName].Service, wsid)
+			assert.Equal(t, workspaceMainConfig.HTTP.Routers[healthzName].Rule, "Path(`/wsid/m1/9999/healthz`)")
+			assert.NotContains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, "wsid"+gateway.AuthMiddlewareSuffix)
+			assert.Contains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, "wsid"+gateway.StripPrefixMiddlewareSuffix)
+			assert.Contains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, "wsid"+gateway.HeaderRewriteMiddlewareSuffix)
+		})
+
+		t.Run("testHealthzEndpointInWorkspaceRoute", func(t *testing.T) {
+			healthzName := "wsid-m1-9999-healthz"
+			assert.Contains(t, workspaceConfig.HTTP.Routers, healthzName)
+			assert.Equal(t, workspaceConfig.HTTP.Routers[healthzName].Service, healthzName)
+			assert.Equal(t, workspaceConfig.HTTP.Routers[healthzName].Rule, "Path(`/m1/9999/healthz`)")
+			assert.NotContains(t, workspaceConfig.HTTP.Routers[healthzName].Middlewares, healthzName+gateway.AuthMiddlewareSuffix)
+			assert.Contains(t, workspaceConfig.HTTP.Routers[healthzName].Middlewares, healthzName+gateway.StripPrefixMiddlewareSuffix)
+		})
+	})
+}
+
+func TestUniqueMainEndpoint(t *testing.T) {
+	wsid := "wsid123"
+
+	infrastructure.InitializeForTesting(infrastructure.OpenShiftv4)
+	cl, _, _ := getSpecObjects(t, &dwo.DevWorkspaceRouting{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "routing",
+			Namespace: "ws",
+		},
+		Spec: dwo.DevWorkspaceRoutingSpec{
+			DevWorkspaceId: wsid,
+			RoutingClass:   "che",
+			Endpoints: map[string]dwo.EndpointList{
+				"m1": {
+					{
+						Name:       "e1",
+						TargetPort: 9999,
+						Exposure:   dw.PublicEndpointExposure,
+						Protocol:   "https",
+						Path:       "/1/",
+						Attributes: attributes.Attributes{
+							urlRewriteSupportedEndpointAttributeName: apiext.JSON{Raw: []byte("\"true\"")},
+							string(dwo.TypeEndpointAttribute):        apiext.JSON{Raw: []byte("\"main\"")},
+							uniqueEndpointAttributeName:              apiext.JSON{Raw: []byte("\"true\"")},
+						},
+					},
+				},
+			},
+		},
+	})
+
+	cms := &corev1.ConfigMapList{}
+	cl.List(context.TODO(), cms)
+
+	assert.Len(t, cms.Items, 2)
+
+	var workspaceMainCfg *corev1.ConfigMap
+	var workspaceCfg *corev1.ConfigMap
+	for _, cfg := range cms.Items {
+		if cfg.Name == wsid+"-route" && cfg.Namespace == "ns" {
+			workspaceMainCfg = cfg.DeepCopy()
+		}
+		if cfg.Name == wsid+"-route" && cfg.Namespace == "ws" {
+			workspaceCfg = cfg.DeepCopy()
+		}
+	}
+
+	traefikWorkspaceConfig := workspaceCfg.Data["workspace.yml"]
+	workspaceConfig := gateway.TraefikConfig{}
+	assert.NoError(t, yaml.Unmarshal([]byte(traefikWorkspaceConfig), &workspaceConfig))
+
+	traefikMainWorkspaceConfig := workspaceMainCfg.Data[wsid+".yml"]
+	workspaceMainConfig := gateway.TraefikConfig{}
+	assert.NoError(t, yaml.Unmarshal([]byte(traefikMainWorkspaceConfig), &workspaceMainConfig))
+
+	t.Run("testHealthzEndpointInMainWorkspaceRoute", func(t *testing.T) {
+		healthzName := "e1-healthz"
+		assert.Contains(t, workspaceMainConfig.HTTP.Routers, healthzName)
+		assert.Equal(t, workspaceMainConfig.HTTP.Routers[healthzName].Service, wsid)
+		assert.Equal(t, workspaceMainConfig.HTTP.Routers[healthzName].Rule, "Path(`/"+wsid+"/m1/e1/healthz`)")
+		assert.NotContains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, wsid+gateway.AuthMiddlewareSuffix)
+		assert.Contains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, wsid+gateway.StripPrefixMiddlewareSuffix)
+		assert.Contains(t, workspaceMainConfig.HTTP.Routers[healthzName].Middlewares, wsid+gateway.HeaderRewriteMiddlewareSuffix)
+	})
+
+	t.Run("testHealthzEndpointInWorkspaceRoute", func(t *testing.T) {
+		healthzName := wsid + "-m1-e1-healthz"
+		assert.Contains(t, workspaceConfig.HTTP.Routers, healthzName)
+		assert.Equal(t, workspaceConfig.HTTP.Routers[healthzName].Service, healthzName)
+		assert.Equal(t, workspaceConfig.HTTP.Routers[healthzName].Rule, "Path(`/m1/e1/healthz`)")
+		assert.NotContains(t, workspaceConfig.HTTP.Routers[healthzName].Middlewares, healthzName+gateway.AuthMiddlewareSuffix)
+		assert.Contains(t, workspaceConfig.HTTP.Routers[healthzName].Middlewares, healthzName+gateway.StripPrefixMiddlewareSuffix)
+	})
+}
+
 func TestCreateSubDomainObjects(t *testing.T) {
 	testCommon := func(infra infrastructure.Type) solvers.RoutingObjects {
 		util.IsOpenShift = infra == infrastructure.OpenShiftv4
@@ -302,9 +499,12 @@ func TestCreateSubDomainObjects(t *testing.T) {
 
 		cl, _, objs := getSpecObjects(t, subdomainDevWorkspaceRouting())
 
-		t.Run("noPodAdditions", func(t *testing.T) {
-			if objs.PodAdditions != nil {
-				t.Error()
+		t.Run("testPodAdditions", func(t *testing.T) {
+			if len(objs.PodAdditions.Containers) != 1 || objs.PodAdditions.Containers[0].Name != wsGatewayName {
+				t.Error("expected Container pod addition with Workspace Gateway. Got ", objs.PodAdditions)
+			}
+			if len(objs.PodAdditions.Volumes) != 1 || objs.PodAdditions.Volumes[0].Name != wsGatewayName {
+				t.Error("expected Volume pod addition for workspace gateway. Got ", objs.PodAdditions)
 			}
 		})
 
@@ -329,8 +529,8 @@ func TestCreateSubDomainObjects(t *testing.T) {
 			cms := &corev1.ConfigMapList{}
 			cl.List(context.TODO(), cms)
 
-			if len(cms.Items) != 0 {
-				t.Errorf("there should be 0 configmaps created but there were: %d", len(cms.Items))
+			if len(cms.Items) != 2 {
+				t.Errorf("there should be 2 configmaps create but found: %d", len(cms.Items))
 			}
 		})
 
@@ -339,26 +539,33 @@ func TestCreateSubDomainObjects(t *testing.T) {
 
 	t.Run("expectedIngresses", func(t *testing.T) {
 		objs := testCommon(infrastructure.Kubernetes)
-		if len(objs.Ingresses) != 1 {
-			t.Error()
+		if len(objs.Ingresses) != 3 {
+			t.Error("Expected 3 ingress, found ", len(objs.Ingresses))
 		}
 		if objs.Ingresses[0].Spec.Rules[0].Host != "wsid-1.down.on.earth" {
-			t.Error()
+			t.Error("Expected Ingress host 'wsid-1.down.on.earth', but got ", objs.Ingresses[0].Spec.Rules[0].Host)
+		}
+		if objs.Ingresses[1].Spec.Rules[0].Host != "wsid-2.down.on.earth" {
+			t.Error("Expected Ingress host 'wsid-2.down.on.earth', but got ", objs.Ingresses[1].Spec.Rules[0].Host)
+		}
+		if objs.Ingresses[2].Spec.Rules[0].Host != "wsid-3.down.on.earth" {
+			t.Error("Expected Ingress host 'wsid-3.down.on.earth', but got ", objs.Ingresses[2].Spec.Rules[0].Host)
 		}
 	})
 
 	t.Run("expectedRoutes", func(t *testing.T) {
 		objs := testCommon(infrastructure.OpenShiftv4)
-		if len(objs.Routes) != 1 {
-			t.Error()
+		if len(objs.Routes) != 3 {
+			t.Error("Expected 3 Routes, found ", len(objs.Routes))
 		}
 		if objs.Routes[0].Spec.Host != "wsid-1.down.on.earth" {
-			t.Error()
+			t.Error("Expected Route host 'wsid-1.down.on.earth', but got ", objs.Routes[0].Spec.Host)
 		}
 	})
 }
 
 func TestReportRelocatableExposedEndpoints(t *testing.T) {
+	// kubernetes
 	util.IsOpenShift = false
 	util.IsOpenShift4 = false
 
@@ -415,7 +622,6 @@ func TestReportRelocatableExposedEndpoints(t *testing.T) {
 func TestReportSubdomainExposedEndpoints(t *testing.T) {
 	util.IsOpenShift = false
 	util.IsOpenShift4 = false
-
 	routing := subdomainDevWorkspaceRouting()
 	_, solver, objs := getSpecObjects(t, routing)
 
@@ -453,23 +659,22 @@ func TestReportSubdomainExposedEndpoints(t *testing.T) {
 	if e2.Name != "e2" {
 		t.Errorf("The second endpoint should have been e2 but is %s", e1.Name)
 	}
-	if e2.Url != "https://wsid-1.down.on.earth/2.js" {
-		t.Errorf("The e2 endpoint should have the following URL: '%s' but has '%s'.", "https://wsid-1.down.on.earth/2.js", e2.Url)
+	if e2.Url != "https://wsid-2.down.on.earth/2.js" {
+		t.Errorf("The e2 endpoint should have the following URL: '%s' but has '%s'.", "https://wsid-2.down.on.earth/2.js", e2.Url)
 	}
 
 	e3 := m1[2]
 	if e3.Name != "e3" {
 		t.Errorf("The third endpoint should have been e3 but is %s", e1.Name)
 	}
-	if e3.Url != "http://wsid-1.down.on.earth/" {
-		t.Errorf("The e3 endpoint should have the following URL: '%s' but has '%s'.", "https://wsid-1.down.on.earth/", e3.Url)
+	if e3.Url != "http://wsid-3.down.on.earth/" {
+		t.Errorf("The e3 endpoint should have the following URL: '%s' but has '%s'.", "https://wsid-3.down.on.earth/", e3.Url)
 	}
 }
 
 func TestFinalize(t *testing.T) {
 	util.IsOpenShift = false
 	util.IsOpenShift4 = false
-
 	routing := relocatableDevWorkspaceRouting()
 	cl, slv, _ := getSpecObjects(t, routing)
 
@@ -492,7 +697,6 @@ func TestFinalize(t *testing.T) {
 func TestEndpointsAlwaysOnSecureProtocolsWhenExposedThroughGateway(t *testing.T) {
 	util.IsOpenShift = false
 	util.IsOpenShift4 = false
-
 	routing := relocatableDevWorkspaceRouting()
 	_, slv, objs := getSpecObjects(t, routing)
 
@@ -541,7 +745,7 @@ func TestUsesIngressAnnotationsForWorkspaceEndpointIngresses(t *testing.T) {
 
 	_, _, objs := getSpecObjectsForManager(t, mgr, subdomainDevWorkspaceRouting())
 
-	if len(objs.Ingresses) != 1 {
+	if len(objs.Ingresses) != 3 {
 		t.Fatalf("Unexpected number of generated ingresses: %d", len(objs.Ingresses))
 	}
 
@@ -588,7 +792,7 @@ func TestUsesCustomCertificateForWorkspaceEndpointIngresses(t *testing.T) {
 		},
 	})
 
-	if len(objs.Ingresses) != 1 {
+	if len(objs.Ingresses) != 3 {
 		t.Fatalf("Unexpected number of generated ingresses: %d", len(objs.Ingresses))
 	}
 
@@ -609,6 +813,30 @@ func TestUsesCustomCertificateForWorkspaceEndpointIngresses(t *testing.T) {
 	if ingress.Spec.TLS[0].Hosts[0] != "wsid-1.almost.trivial" {
 		t.Errorf("Unexpected host name of the TLS spec: %s", ingress.Spec.TLS[0].Hosts[0])
 	}
+
+	ingress = objs.Ingresses[1]
+
+	if len(ingress.Spec.TLS) != 1 {
+		t.Fatalf("Unexpected number of TLS records on the ingress: %d", len(ingress.Spec.TLS))
+	}
+
+	if ingress.Spec.TLS[0].SecretName != "wsid-endpoints" {
+		t.Errorf("Unexpected name of the TLS secret on the ingress: %s", ingress.Spec.TLS[0].SecretName)
+	}
+
+	if len(ingress.Spec.TLS[0].Hosts) != 1 {
+		t.Fatalf("Unexpected number of host records on the TLS spec: %d", len(ingress.Spec.TLS[0].Hosts))
+	}
+
+	if ingress.Spec.TLS[0].Hosts[0] != "wsid-2.almost.trivial" {
+		t.Errorf("Unexpected host name of the TLS spec: %s", ingress.Spec.TLS[0].Hosts[0])
+	}
+
+	ingress = objs.Ingresses[2]
+
+	if len(ingress.Spec.TLS) != 0 {
+		t.Fatalf("Unexpected number of TLS records on the ingress: %d", len(ingress.Spec.TLS))
+	}
 }
 
 func TestUsesCustomCertificateForWorkspaceEndpointRoutes(t *testing.T) {
@@ -643,7 +871,7 @@ func TestUsesCustomCertificateForWorkspaceEndpointRoutes(t *testing.T) {
 		},
 	})
 
-	if len(objs.Routes) != 1 {
+	if len(objs.Routes) != 3 {
 		t.Fatalf("Unexpected number of generated routes: %d", len(objs.Routes))
 	}
 
@@ -656,6 +884,22 @@ func TestUsesCustomCertificateForWorkspaceEndpointRoutes(t *testing.T) {
 	if route.Spec.TLS.Key != "asdf" {
 		t.Errorf("Unexpected key of TLS spec: %s", route.Spec.TLS.Key)
 	}
+
+	route = objs.Routes[1]
+
+	if route.Spec.TLS.Certificate != "qwer" {
+		t.Errorf("Unexpected name of the TLS certificate on the route: %s", route.Spec.TLS.Certificate)
+	}
+
+	if route.Spec.TLS.Key != "asdf" {
+		t.Errorf("Unexpected key of TLS spec: %s", route.Spec.TLS.Key)
+	}
+
+	route = objs.Routes[2]
+
+	if route.Spec.TLS != nil {
+		t.Errorf("Unexpected TLS on the route: %s", route.Spec.TLS)
+	}
 }
 
 func asV1(v2Obj *v2alpha1.CheCluster) *v1.CheCluster {
