@@ -10,6 +10,8 @@ forceBuild=0
 # so that rhpkg build is simply a brew wrapper (using get-sources.sh -f)
 pullAssets=0
 
+SCRIPT_DIR=$(dirname $(readlink -f "${BASH_SOURCE[0]}"))
+
 DEV_WORKSPACE_CONTROLLER_VERSION="" # main or 0.y.x
 DEV_HEADER_REWRITE_TRAEFIK_PLUGIN="" # main or v0.y.z
 RESTIC_VERSION=$(sed -n 's|.*RESTIC_TAG=\(.*\)|\1|p' Dockerfile)
@@ -40,36 +42,9 @@ function logn()
 }
 
 if [[ ${pullAssets} -eq 1 ]]; then 
-	# if not set via commandline, compute DEV_WORKSPACE_CONTROLLER_VERSION and DEV_HEADER_REWRITE_TRAEFIK_PLUGIN
-	# from https://raw.githubusercontent.com/redhat-developer/codeready-workspaces/crw-2-rhel-8/dependencies/job-config.json
-	# shellcheck disable=SC2086
-	if [[ -z "${DEV_WORKSPACE_CONTROLLER_VERSION}" ]] || [[ -z "${DEV_HEADER_REWRITE_TRAEFIK_PLUGIN}" ]]; then
-		MIDSTM_BRANCH=$(git rev-parse --abbrev-ref HEAD 2>/dev/null || echo "crw-2-rhel-8")
-		if [[ ${MIDSTM_BRANCH} != "crw-"*"-rhel-"* ]]; then MIDSTM_BRANCH="crw-2-rhel-8"; fi
-		configjson="$(curl -sSLo- "https://raw.githubusercontent.com/redhat-developer/codeready-workspaces/${MIDSTM_BRANCH}/dependencies/job-config.json")"
-		if [[ $configjson == *"404"* ]] || [[ $configjson == *"Not Found"* ]]; then 
-			echo "[ERROR] Could not load https://raw.githubusercontent.com/redhat-developer/codeready-workspaces/${MIDSTM_BRANCH}/dependencies/job-config.json"
-			echo "[ERROR] Please use --dwob flag to set DEV_WORKSPACE_CONTROLLER_VERSION"
-			echo "[ERROR] Please use --hrtpb flag to set DEV_HEADER_REWRITE_TRAEFIK_PLUGIN"
-			exit 1
-		fi
-		if [[ $MIDSTM_BRANCH == "crw-2-rhel-8" ]]; then
-			CRW_VERSION="$(echo "$configjson" | jq -r '.Version')"
-		else 
-			CRW_VERSION=${MIDSTM_BRANCH/crw-/}; CRW_VERSION=${CRW_VERSION//-rhel-8}
-		fi
-		if [[ -z "${DEV_WORKSPACE_CONTROLLER_VERSION}" ]]; then
-			DEV_WORKSPACE_CONTROLLER_VERSION="$(echo "$configjson" | jq -r '.Jobs["devworkspace-controller"]["'${CRW_VERSION}'"].upstream_branch[0]')"
-			if [[ ${DEV_WORKSPACE_CONTROLLER_VERSION} == "null" ]]; then DEV_WORKSPACE_CONTROLLER_VERSION="main"; fi
-		fi
-		if [[ -z "${DEV_HEADER_REWRITE_TRAEFIK_PLUGIN}" ]]; then
-			DEV_HEADER_REWRITE_TRAEFIK_PLUGIN="$(echo "$configjson" | jq -r '.Other["DEV_HEADER_REWRITE_TRAEFIK_PLUGIN"]["'${CRW_VERSION}'"]')"
-			if [[ ${DEV_HEADER_REWRITE_TRAEFIK_PLUGIN} == "null" ]]; then DEV_HEADER_REWRITE_TRAEFIK_PLUGIN="main"; fi
-		fi
-	fi
-	echo "[INFO] For ${CRW_VERSION} / ${MIDSTM_BRANCH}:"
-	echo "[INFO]   DEV_WORKSPACE_CONTROLLER_VERSION   = ${DEV_WORKSPACE_CONTROLLER_VERSION}"
-	echo "[INFO]   DEV_HEADER_REWRITE_TRAEFIK_PLUGIN  = ${DEV_HEADER_REWRITE_TRAEFIK_PLUGIN}"
+	if [[ -x "${SCRIPT_DIR}"/build/scripts/util.sh ]]; then  source "${SCRIPT_DIR}"/build/scripts/util.sh;
+	else echo "Error: can't find build/scripts/util.sh in ${SCRIPT_DIR}"; exit 1; fi
+	updateDockerfileArgs Dockerfile Dockerfile.2
 
 	set -x
 	# step 1 - get zips & repack them
@@ -104,12 +79,6 @@ if [[ ${pullAssets} -eq 1 ]]; then
 	${BUILDER} rmi "${tag}:bootstrap"
 	set +x
 
-	# update Dockerfile to record version we expect for DEV_WORKSPACE_CONTROLLER_VERSION and DEV_HEADER_REWRITE_TRAEFIK_PLUGIN
-	# CRW-1674 this step also done in crw-operator_2.*.jenkinsfile
-	sed Dockerfile -r \
-		-e 's#DEV_WORKSPACE_CONTROLLER_VERSION="([^"]+)"#DEV_WORKSPACE_CONTROLLER_VERSION="'${DEV_WORKSPACE_CONTROLLER_VERSION}'"#' \
-		-e 's#DEV_HEADER_REWRITE_TRAEFIK_PLUGIN="([^"]+)"#DEV_HEADER_REWRITE_TRAEFIK_PLUGIN="'${DEV_HEADER_REWRITE_TRAEFIK_PLUGIN}'"#' \
-		> Dockerfile.2
 else
 	cp Dockerfile Dockerfile.2
 fi
