@@ -15,24 +15,31 @@ package main
 import (
 	"flag"
 	"os"
+	"time"
+
+	"github.com/devfile/devworkspace-operator/pkg/infrastructure"
+
+	dwoApi "github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
+	dwr "github.com/devfile/devworkspace-operator/controllers/controller/devworkspacerouting"
+	"github.com/eclipse-che/che-operator/controllers/devworkspace"
+	"github.com/eclipse-che/che-operator/controllers/devworkspace/solver"
+
+	"github.com/eclipse-che/che-operator/controllers/usernamespace"
 
 	"go.uber.org/zap/zapcore"
 	// Import all Kubernetes client auth plugins (e.g. Azure, GCP, OIDC, etc.)
 	// to ensure that exec-entrypoint and run can make use of them.
-
+	"k8s.io/client-go/discovery"
 	_ "k8s.io/client-go/plugin/pkg/client/auth"
 
 	"k8s.io/apimachinery/pkg/runtime"
 	ctrl "sigs.k8s.io/controller-runtime"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/healthz"
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
 
 	osruntime "runtime"
 
-	dwo_api "github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
-	dwr "github.com/devfile/devworkspace-operator/controllers/controller/devworkspacerouting"
-	"github.com/devfile/devworkspace-operator/pkg/infrastructure"
-
 	"fmt"
 
 	"github.com/go-logr/logr"
@@ -44,8 +51,6 @@ import (
 	checontroller "github.com/eclipse-che/che-operator/controllers/che"
 	backupcontroller "github.com/eclipse-che/che-operator/controllers/checlusterbackup"
 	restorecontroller "github.com/eclipse-che/che-operator/controllers/checlusterrestore"
-	"github.com/eclipse-che/che-operator/controllers/devworkspace"
-	"github.com/eclipse-che/che-operator/controllers/devworkspace/solver"
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 	"github.com/eclipse-che/che-operator/pkg/signal"
 	"github.com/eclipse-che/che-operator/pkg/util"
@@ -60,7 +65,8 @@ import (
 	rbac "k8s.io/api/rbac/v1"
 	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 
-	image_puller_api "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis"
+	image_puller_api "github.com/che-incubator/kubernetes-image-puller-operator/api/v1alpha1"
+	projectv1 "github.com/openshift/api/project/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	userv1 "github.com/openshift/api/user/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -76,6 +82,13 @@ var (
 	metricsAddr          string
 	enableLeaderElection bool
 	probeAddr            string
+
+	leaseDuration = 40 * time.Second
+	renewDeadline = 30 * time.Second
+)
+
+const (
+	leasesApiResourceName = "leases"
 )
 
 func init() {
@@ -129,6 +142,7 @@ func init() {
 		utilruntime.Must(configv1.AddToScheme(scheme))
 		utilruntime.Must(corev1.AddToScheme(scheme))
 		utilruntime.Must(consolev1.AddToScheme(scheme))
+		utilruntime.Must(projectv1.AddToScheme(scheme))
 	}
 }
 
@@ -189,31 +203,48 @@ func main() {
 			"the manager will watch and manage resources in all namespaces")
 	}
 
-	mgr, err := ctrl.NewManager(ctrl.GetConfigOrDie(), ctrl.Options{
-		Scheme:                 scheme,
-		MetricsBindAddress:     metricsAddr,
-		Port:                   9443,
-		HealthProbeBindAddress: probeAddr,
-		LeaderElection:         enableLeaderElection,
-		LeaderElectionID:       "e79b08a4.org.eclipse.che",
+	config := ctrl.GetConfigOrDie()
+
+	discoveryClient, err := discovery.NewDiscoveryClientForConfig(config)
+	if err != nil {
+		setupLog.Error(err, "failed to create discovery client")
+		os.Exit(1)
+	}
+
+	if !util.HasK8SResourceObject(discoveryClient, leasesApiResourceName) {
+		setupLog.Info("Leader election was disabled", "Cause:", leasesApiResourceName+"k8s api resource is an absent.")
+		enableLeaderElection = false
+	}
+
+	mgr, err := ctrl.NewManager(config, ctrl.Options{
+		Scheme:                        scheme,
+		MetricsBindAddress:            metricsAddr,
+		Port:                          9443,
+		HealthProbeBindAddress:        probeAddr,
+		LeaderElection:                enableLeaderElection,
+		LeaderElectionID:              "e79b08a4.org.eclipse.che",
+		LeaderElectionReleaseOnCancel: true,
+		LeaseDuration:                 &leaseDuration,
+		RenewDeadline:                 &renewDeadline,
+
 		// NOTE: We CANNOT limit the manager to a single namespace, because that would limit the
 		// devworkspace routing reconciler to a single namespace, which would make it totally unusable.
 		// Instead, if some controller wants to limit itself to single namespace, it can do it
 		// for example using an event filter, as checontroller does.
 		// Namespace:              watchNamespace,
-		// TODO try to use it instead of signal handler....
-		// GracefulShutdownTimeout: ,
 	})
 	if err != nil {
 		setupLog.Error(err, "unable to start manager")
 		os.Exit(1)
 	}
 
-	cheReconciler, err := checontroller.NewReconciler(mgr, watchNamespace)
+	noncachedClient, err := client.New(mgr.GetConfig(), client.Options{Scheme: scheme})
 	if err != nil {
-		setupLog.Error(err, "unable to create checluster reconciler")
+		setupLog.Error(err, "unable to initialize non cached client")
 		os.Exit(1)
 	}
+
+	cheReconciler := checontroller.NewReconciler(mgr.GetClient(), noncachedClient, discoveryClient, mgr.GetScheme(), watchNamespace)
 	backupReconciler := backupcontroller.NewReconciler(mgr, watchNamespace)
 	restoreReconciler := restorecontroller.NewReconciler(mgr, watchNamespace)
 
@@ -247,7 +278,7 @@ func main() {
 		os.Exit(1)
 	}
 	if shouldDevWorkspaceBeEnabled {
-		if err := dwo_api.AddToScheme(mgr.GetScheme()); err != nil {
+		if err := dwoApi.AddToScheme(mgr.GetScheme()); err != nil {
 			setupLog.Error(err, "failed to register DevWorkspace API Scheme", "controller", "DevWorkspace")
 			os.Exit(1)
 		}
@@ -269,10 +300,15 @@ func main() {
 			os.Exit(1)
 		}
 
+		userNamespaceReconciler := usernamespace.NewReconciler()
+		if err = userNamespaceReconciler.SetupWithManager(mgr); err != nil {
+			setupLog.Error(err, "unable to set up controller", "controller", "CheUserReconciler")
+			os.Exit(1)
+		}
 		setupLog.Info("DevWorkspace support enabled.")
 	} else {
 		setupLog.Info("DevWorkspace support disabled. Will initiate restart when CheCluster with devworkspaces enabled will appear")
-		go devworkspace.NotifyWhenDevWorkspaceEnabled(mgr, sigHandler, func() {
+		go devworkspace.NotifyWhenDevWorkspaceEnabled(mgr, sigHandler.Done(), func() {
 			setupLog.Info("CheCluster with DevWorkspace enabled discovered. Restarting to activate DevWorkspaces mode")
 			os.Exit(0)
 		})
