@@ -9,16 +9,30 @@
 // Contributors:
 //   Red Hat, Inc. - initial API and implementation
 //
+
 package dashboard
 
 import (
+	"fmt"
+
+	ctrl "sigs.k8s.io/controller-runtime"
+
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 	"github.com/eclipse-che/che-operator/pkg/deploy/expose"
+	"github.com/eclipse-che/che-operator/pkg/deploy/gateway"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/types"
 )
 
+const (
+	exposePath = "/dashboard/"
+)
+
+var (
+	log = ctrl.Log.WithName("dashboard")
+)
+
 type Dashboard struct {
 	deployContext *deploy.DeployContext
 	component     string
@@ -44,9 +58,10 @@ func (d *Dashboard) Reconcile() (done bool, err error) {
 
 	// Expose dashboard service with route or ingress
 	_, done, err = expose.ExposeWithHostPath(d.deployContext, d.component, d.deployContext.CheCluster.Spec.Server.CheHost,
-		"/dashboard/",
+		exposePath,
 		d.deployContext.CheCluster.Spec.Server.DashboardRoute,
 		d.deployContext.CheCluster.Spec.Server.DashboardIngress,
+		d.createGatewayConfig(),
 	)
 	if !done {
 		return false, err
@@ -55,8 +70,9 @@ func (d *Dashboard) Reconcile() (done bool, err error) {
 	// we create dashboard SA in any case to keep a track on resources we access withing it
 	done, err = deploy.SyncServiceAccountToCluster(d.deployContext, DashboardSA)
 	if !done {
-		return false, err
+		return done, err
 	}
+
 	// on Kubernetes Dashboard needs privileged SA to work with user's objects
 	// for time being until Kubernetes did not get authentication
 	if !util.IsOpenShift {
@@ -98,3 +114,16 @@ func (d *Dashboard) Finalize() (done bool, err error) {
 	err = deploy.DeleteFinalizer(d.deployContext, ClusterPermissionsDashboardFinalizer)
 	return err == nil, err
 }
+
+func (d *Dashboard) createGatewayConfig() *gateway.TraefikConfig {
+	cfg := gateway.CreateCommonTraefikConfig(
+		d.component,
+		fmt.Sprintf("PathPrefix(`%s`)", exposePath),
+		10,
+		"http://"+d.component+":8080",
+		[]string{})
+	if util.IsNativeUserModeEnabled(d.deployContext.CheCluster) {
+		cfg.AddAuthHeaderRewrite(d.component)
+	}
+	return cfg
+}
