@@ -27,8 +27,8 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
-	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 
 	"testing"
 )
@@ -47,7 +47,7 @@ var (
 			Template: corev1.PodTemplateSpec{
 				Spec: corev1.PodSpec{
 					Containers: []corev1.Container{
-						corev1.Container{},
+						{},
 					},
 				},
 			},
@@ -68,7 +68,8 @@ func TestMountSecret(t *testing.T) {
 			name: "Mount secret as file",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -80,7 +81,8 @@ func TestMountSecret(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -137,7 +139,8 @@ func TestMountSecret(t *testing.T) {
 			name: "Mount env variable",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -149,7 +152,8 @@ func TestMountSecret(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -203,7 +207,8 @@ func TestMountSecret(t *testing.T) {
 			name: "Mount several env variables",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -215,7 +220,8 @@ func TestMountSecret(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -295,7 +301,7 @@ func TestMountSecret(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 			orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
 			testCase.initObjects = append(testCase.initObjects, testCase.initDeployment)
 			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
@@ -338,7 +344,8 @@ func TestMountConfigMaps(t *testing.T) {
 			name: "Mount configmap as file",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -350,7 +357,8 @@ func TestMountConfigMaps(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -409,7 +417,8 @@ func TestMountConfigMaps(t *testing.T) {
 			name: "Mount env variable",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -421,7 +430,8 @@ func TestMountConfigMaps(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -475,7 +485,8 @@ func TestMountConfigMaps(t *testing.T) {
 			name: "Mount several env variables",
 			initDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -487,7 +498,8 @@ func TestMountConfigMaps(t *testing.T) {
 			},
 			expectedDeployment: &appsv1.Deployment{
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "che",
+					Name:            "che",
+					ResourceVersion: "0",
 				},
 				Spec: appsv1.DeploymentSpec{
 					Template: corev1.PodTemplateSpec{
@@ -567,7 +579,7 @@ func TestMountConfigMaps(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 			orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
 			testCase.initObjects = append(testCase.initObjects, testCase.initDeployment)
 			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
