@@ -17,7 +17,7 @@ import (
 	"strings"
 	"time"
 
-	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
+	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/api/v1alpha1"
 	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	operatorsv1 "github.com/operator-framework/api/pkg/operators/v1"
@@ -28,10 +28,15 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/validation"
+	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
 
+var (
+	log = ctrl.Log.WithName("image-puller")
+)
+
 var imagePullerFinalizerName = "kubernetesimagepullers.finalizers.che.eclipse.org"
 
 // ImageAndName represents an image coupled with an image name.
@@ -40,21 +45,35 @@ type ImageAndName struct {
 	Image string // image (ex. quay.io/test/abc)
 }
 
+type ImagePuller struct {
+}
+
+func NewImagePuller() *ImagePuller {
+	return &ImagePuller{}
+}
+
+func (ip *ImagePuller) Reconcile(ctx *DeployContext) (reconcile.Result, bool, error) {
+	return ReconcileImagePuller(ctx)
+}
+
+func (ip *ImagePuller) Finalize(ctx *DeployContext) (bool, error) {
+	return DeleteImagePullerOperatorAndFinalizer(ctx)
+}
+
 // Reconcile the imagePuller section of the CheCluster CR.  If imagePuller.enable is set to true, install the Kubernetes Image Puller operator and create
 // a KubernetesImagePuller CR.  Add a finalizer to the CheCluster CR.  If false, remove the KubernetesImagePuller CR, uninstall the operator, and remove the finalizer.
-func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
-
+func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, bool, error) {
 	// Determine what server groups the API Server knows about
 	foundPackagesAPI, foundOperatorsAPI, _, err := CheckNeededImagePullerApis(ctx)
 	if err != nil {
 		logrus.Errorf("Error discovering image puller APIs: %v", err)
-		return reconcile.Result{}, err
+		return reconcile.Result{}, false, err
 	}
 
 	// If the image puller should be installed but the APIServer doesn't know about PackageManifests/Subscriptions, log a warning and requeue
 	if ctx.CheCluster.Spec.ImagePuller.Enable && (!foundPackagesAPI || !foundOperatorsAPI) {
 		logrus.Infof("Couldn't find Operator Lifecycle Manager types to install the Kubernetes Image Puller Operator.  Please install Operator Lifecycle Manager to install the operator or disable the image puller by setting spec.imagePuller.enable to false.")
-		return reconcile.Result{RequeueAfter: time.Second}, nil
+		return reconcile.Result{RequeueAfter: time.Second}, false, nil
 	}
 
 	if ctx.CheCluster.Spec.ImagePuller.Enable {
@@ -63,42 +82,42 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 			if err != nil {
 				if errors.IsNotFound(err) {
 					logrus.Infof("There is no PackageManifest for the Kubernetes Image Puller Operator.  Install the Operator Lifecycle Manager and the Community Operators Catalog")
-					return reconcile.Result{RequeueAfter: time.Second}, nil
+					return reconcile.Result{RequeueAfter: time.Second}, false, nil
 				}
 				logrus.Errorf("Error getting packagemanifest: %v", err)
-				return reconcile.Result{}, err
+				return reconcile.Result{}, false, err
 			}
 
 			createdOperatorGroup, err := CreateOperatorGroupIfNotFound(ctx)
 			if err != nil {
 				logrus.Infof("Error creating OperatorGroup: %v", err)
-				return reconcile.Result{}, err
+				return reconcile.Result{}, false, err
 			}
 			if createdOperatorGroup {
-				return reconcile.Result{RequeueAfter: time.Second}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, false, nil
 			}
 			createdOperatorSubscription, err := CreateImagePullerSubscription(ctx, packageManifest)
 			if err != nil {
 				logrus.Infof("Error creating Subscription: %v", err)
-				return reconcile.Result{}, err
+				return reconcile.Result{}, false, err
 			}
 			if createdOperatorSubscription {
-				return reconcile.Result{RequeueAfter: time.Second}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, false, nil
 			}
 
 			// Add the image puller finalizer
 			if !HasImagePullerFinalizer(ctx.CheCluster) {
 				if err := ReconcileImagePullerFinalizer(ctx); err != nil {
-					return reconcile.Result{}, err
+					return reconcile.Result{}, false, err
 				}
-				return reconcile.Result{RequeueAfter: time.Second}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, false, nil
 			}
 		}
 
 		_, _, foundKubernetesImagePullerAPI, err := CheckNeededImagePullerApis(ctx)
 		if err != nil {
 			logrus.Errorf("Error discovering image puller APIs: %v", err)
-			return reconcile.Result{}, err
+			return reconcile.Result{}, false, err
 		}
 		// If the KubernetesImagePuller API service exists, attempt to reconcile creation/update
 		if foundKubernetesImagePullerAPI {
@@ -116,15 +135,15 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 						_, err := UpdateImagePullerSpecIfEmpty(ctx)
 						if err != nil {
 							logrus.Errorf("Error updating CheCluster: %v", err)
-							return reconcile.Result{}, err
+							return reconcile.Result{}, false, err
 						}
-						return reconcile.Result{RequeueAfter: time.Second}, nil
+						return reconcile.Result{RequeueAfter: time.Second}, false, nil
 					}
 
 					if ctx.CheCluster.IsImagePullerImagesEmpty() {
 						if err = SetDefaultImages(ctx); err != nil {
 							logrus.Error(err)
-							return reconcile.Result{}, err
+							return reconcile.Result{}, false, err
 						}
 					}
 
@@ -132,19 +151,19 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 					createdImagePuller, err := CreateKubernetesImagePuller(ctx)
 					if err != nil {
 						logrus.Error("Error creating KubernetesImagePuller: ", err)
-						return reconcile.Result{}, err
+						return reconcile.Result{}, false, err
 					}
 					if createdImagePuller {
-						return reconcile.Result{}, nil
+						return reconcile.Result{}, false, nil
 					}
 				}
 				logrus.Errorf("Error getting KubernetesImagePuller: %v", err)
-				return reconcile.Result{}, err
+				return reconcile.Result{}, false, err
 			}
 
 			if err = UpdateDefaultImagesIfNeeded(ctx); err != nil {
 				logrus.Error(err)
-				return reconcile.Result{}, err
+				return reconcile.Result{}, false, err
 			}
 
 			if ctx.CheCluster.Spec.ImagePuller.Spec.DeploymentName == "" {
@@ -153,6 +172,9 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 			if ctx.CheCluster.Spec.ImagePuller.Spec.ConfigMapName == "" {
 				ctx.CheCluster.Spec.ImagePuller.Spec.ConfigMapName = imagePuller.Spec.ConfigMapName
 			}
+			if ctx.CheCluster.Spec.ImagePuller.Spec.ImagePullerImage == "" {
+				ctx.CheCluster.Spec.ImagePuller.Spec.ImagePullerImage = imagePuller.Spec.ImagePullerImage
+			}
 
 			// If ImagePuller specs are different, update the KubernetesImagePuller CR
 			if imagePuller.Spec != ctx.CheCluster.Spec.ImagePuller.Spec {
@@ -160,38 +182,43 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 				logrus.Infof("Updating KubernetesImagePuller %v", imagePuller.Name)
 				if err = ctx.ClusterAPI.Client.Update(context.TODO(), imagePuller, &client.UpdateOptions{}); err != nil {
 					logrus.Errorf("Error updating KubernetesImagePuller: %v", err)
-					return reconcile.Result{}, err
+					return reconcile.Result{}, false, err
 				}
-				return reconcile.Result{RequeueAfter: time.Second}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, false, nil
 			}
 		} else {
 			logrus.Infof("Waiting 15 seconds for kubernetesimagepullers.che.eclipse.org API")
-			return reconcile.Result{RequeueAfter: 15 * time.Second}, nil
+			return reconcile.Result{RequeueAfter: 15 * time.Second}, false, nil
 		}
 
 	} else {
 		if foundOperatorsAPI && foundPackagesAPI {
-			removed, err := UninstallImagePullerOperator(ctx)
-			if err != nil {
-				logrus.Errorf("Error uninstalling Image Puller: %v", err)
-				return reconcile.Result{}, err
-			}
-
-			if removed {
-				return reconcile.Result{RequeueAfter: time.Second}, nil
-			}
-
-			if HasImagePullerFinalizer(ctx.CheCluster) {
-				err = DeleteImagePullerFinalizer(ctx)
-				if err != nil {
-					logrus.Errorf("Error deleting finalizer: %v", err)
-					return reconcile.Result{}, err
-				}
-				return reconcile.Result{RequeueAfter: time.Second}, nil
+			done, err := DeleteImagePullerOperatorAndFinalizer(ctx)
+			if !done {
+				return reconcile.Result{}, false, err
+			} else {
+				return reconcile.Result{}, true, nil
 			}
 		}
 	}
-	return reconcile.Result{}, nil
+	return reconcile.Result{}, true, nil
+}
+
+func DeleteImagePullerOperatorAndFinalizer(ctx *DeployContext) (bool, error) {
+	if _, err := GetImagePullerOperator(ctx); err == nil {
+		if _, err := UninstallImagePullerOperator(ctx); err != nil {
+			logrus.Errorf("Error uninstalling Image Puller: %v", err)
+			return false, err
+		}
+	}
+	if HasImagePullerFinalizer(ctx.CheCluster) {
+		if err := DeleteImagePullerFinalizer(ctx); err != nil {
+			logrus.Errorf("Error deleting finalizer: %v", err)
+			return false, err
+		}
+	}
+
+	return true, nil
 }
 
 func HasImagePullerFinalizer(instance *orgv1.CheCluster) bool {
@@ -276,7 +303,7 @@ func CheckNeededImagePullerApis(ctx *DeployContext) (bool, bool, bool, error) {
 
 	for _, l := range resourcesList {
 		for _, r := range l.APIResources {
-			if l.GroupVersion == chev1alpha1.SchemeGroupVersion.String() && r.Kind == "KubernetesImagePuller" {
+			if l.GroupVersion == chev1alpha1.SchemeBuilder.GroupVersion.String() && r.Kind == "KubernetesImagePuller" {
 				foundKubernetesImagePullerAPI = true
 			}
 		}
@@ -561,7 +588,6 @@ func SetImages(ctx *DeployContext, images []ImageAndName) error {
 // the image puller spec.  Returns true if the CheCluster was updated
 func UninstallImagePullerOperator(ctx *DeployContext) (bool, error) {
 	updated := false
-
 	_, hasOperatorsAPIs, hasImagePullerAPIs, err := CheckNeededImagePullerApis(ctx)
 	if err != nil {
 		return updated, err
@@ -569,8 +595,7 @@ func UninstallImagePullerOperator(ctx *DeployContext) (bool, error) {
 
 	if hasImagePullerAPIs {
 		// Delete the KubernetesImagePuller
-		imagePuller := &chev1alpha1.KubernetesImagePuller{}
-		err := ctx.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: ctx.CheCluster.Namespace, Name: ctx.CheCluster.Name + "-image-puller"}, imagePuller)
+		imagePuller, err := GetImagePullerOperator(ctx)
 		if err != nil && !errors.IsNotFound(err) {
 			return updated, err
 		}
@@ -629,7 +654,8 @@ func UninstallImagePullerOperator(ctx *DeployContext) (bool, error) {
 	}
 
 	// Update CR to remove imagePullerSpec
-	if ctx.CheCluster.Spec.ImagePuller.Enable || ctx.CheCluster.Spec.ImagePuller.Spec != (chev1alpha1.KubernetesImagePullerSpec{}) {
+	if (ctx.CheCluster.Spec.ImagePuller.Enable || ctx.CheCluster.Spec.ImagePuller.Spec != (chev1alpha1.KubernetesImagePullerSpec{})) &&
+		ctx.CheCluster.ObjectMeta.DeletionTimestamp.IsZero() {
 		ctx.CheCluster.Spec.ImagePuller.Spec = chev1alpha1.KubernetesImagePullerSpec{}
 		logrus.Infof("Updating CheCluster %v to remove image puller spec", ctx.CheCluster.Name)
 		err := ctx.ClusterAPI.Client.Update(context.TODO(), ctx.CheCluster, &client.UpdateOptions{})
@@ -640,3 +666,13 @@ func UninstallImagePullerOperator(ctx *DeployContext) (bool, error) {
 	}
 	return updated, nil
 }
+
+// GetImagePullerOperator returns the current kubernetes-imagepuller-operator if exists
+func GetImagePullerOperator(ctx *DeployContext) (*chev1alpha1.KubernetesImagePuller, error) {
+	imagePuller := &chev1alpha1.KubernetesImagePuller{}
+	err := ctx.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: ctx.CheCluster.Namespace, Name: ctx.CheCluster.Name + "-image-puller"}, imagePuller)
+	if err != nil {
+		return nil, err
+	}
+	return imagePuller, nil
+}
