@@ -14,7 +14,7 @@ package deploy
 import (
 	"strings"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 )
 
 func GetLabels(cheCluster *orgv1.CheCluster, component string) map[string]string {
