@@ -103,9 +103,6 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 		}
 	}
 
-	if err != nil {
-		logrus.Errorf("Failed to get current infra: %s", err)
-	}
 	cheFlavor := deploy.DefaultCheFlavor(s.deployContext.CheCluster)
 	infra := "kubernetes"
 	if util.IsOpenShift {
@@ -181,7 +178,6 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 	cheDebug := util.GetValue(s.deployContext.CheCluster.Spec.Server.CheDebug, deploy.DefaultCheDebug)
 	cheMetrics := strconv.FormatBool(s.deployContext.CheCluster.Spec.Metrics.Enable)
 	cheLabels := util.MapToKeyValuePairs(deploy.GetLabels(s.deployContext.CheCluster, deploy.DefaultCheFlavor(s.deployContext.CheCluster)))
-	cheMultiUser := deploy.GetCheMultiUser(s.deployContext.CheCluster)
 	workspaceExposure := deploy.GetSingleHostExposureType(s.deployContext.CheCluster)
 	singleHostGatewayConfigMapLabels := labels.FormatLabels(util.GetMapValue(s.deployContext.CheCluster.Spec.Server.SingleHostGatewayConfigMapLabels, deploy.DefaultSingleHostGatewayConfigMapLabels))
 	workspaceNamespaceDefault := util.GetWorkspaceNamespaceDefault(s.deployContext.CheCluster)
@@ -214,7 +210,7 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 	webSocketEndpoint := wsprotocol + "://" + cheHost + "/api/websocket"
 
 	data := &CheConfigMap{
-		CheMultiUser:                           cheMultiUser,
+		CheMultiUser:                           "true",
 		CheHost:                                cheHost,
 		ChePort:                                "8080",
 		CheApi:                                 cheAPI,
@@ -257,16 +253,14 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 		CheDevWorkspacesEnabled:                strconv.FormatBool(s.deployContext.CheCluster.Spec.DevWorkspace.Enable),
 	}
 
-	if cheMultiUser == "true" {
-		data.KeycloakURL = keycloakURL
-		data.KeycloakInternalURL = keycloakInternalURL
-		data.KeycloakRealm = keycloakRealm
-		data.KeycloakClientId = keycloakClientId
-		data.DatabaseURL = "jdbc:postgresql://" + chePostgresHostName + ":" + chePostgresPort + "/" + chePostgresDb
-		if len(s.deployContext.CheCluster.Spec.Database.ChePostgresSecret) < 1 {
-			data.DbUserName = s.deployContext.CheCluster.Spec.Database.ChePostgresUser
-			data.DbPassword = s.deployContext.CheCluster.Spec.Database.ChePostgresPassword
-		}
+	data.KeycloakURL = keycloakURL
+	data.KeycloakInternalURL = keycloakInternalURL
+	data.KeycloakRealm = keycloakRealm
+	data.KeycloakClientId = keycloakClientId
+	data.DatabaseURL = "jdbc:postgresql://" + chePostgresHostName + ":" + chePostgresPort + "/" + chePostgresDb
+	if len(s.deployContext.CheCluster.Spec.Database.ChePostgresSecret) < 1 {
+		data.DbUserName = s.deployContext.CheCluster.Spec.Database.ChePostgresUser
+		data.DbPassword = s.deployContext.CheCluster.Spec.Database.ChePostgresPassword
 	}
 
 	out, err := json.Marshal(data)
@@ -314,15 +308,24 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 
 	addMap(cheEnv, s.deployContext.CheCluster.Spec.Server.CustomCheProperties)
 
-	// Update BitBucket endpoints
-	secrets, err := deploy.GetSecrets(s.deployContext, map[string]string{
+	err = setBitbucketEndpoints(s.deployContext, cheEnv)
+	if err != nil {
+		return nil, err
+	}
+
+	return cheEnv, nil
+}
+
+func setBitbucketEndpoints(deployContext *deploy.DeployContext, cheEnv map[string]string) error {
+	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
 		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
 		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
 	}, map[string]string{
 		deploy.CheEclipseOrgOAuthScmServer: "bitbucket",
 	})
+
 	if err != nil {
-		return nil, err
+		return err
 	} else if len(secrets) == 1 {
 		serverEndpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
 		endpoints, exists := cheEnv["CHE_INTEGRATION_BITBUCKET_SERVER__ENDPOINTS"]
@@ -333,7 +336,7 @@ func (s *Server) getCheConfigMapData() (cheEnv map[string]string, err error) {
 		}
 	}
 
-	return cheEnv, nil
+	return nil
 }
 
 func GetCheConfigMapVersion(deployContext *deploy.DeployContext) string {
