@@ -17,6 +17,8 @@ import (
 	"strings"
 
 	"github.com/eclipse-che/che-operator/pkg/deploy"
+	identity_provider "github.com/eclipse-che/che-operator/pkg/deploy/identity-provider"
+	"github.com/eclipse-che/che-operator/pkg/deploy/postgres"
 
 	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/util"
@@ -27,7 +29,7 @@ import (
 )
 
 func (s Server) getDeploymentSpec() (*appsv1.Deployment, error) {
-	selfSignedCertUsed, err := deploy.IsSelfSignedCertificateUsed(s.deployContext)
+	selfSignedCASecretExists, err := deploy.IsSelfSignedCASecretExists(s.deployContext)
 	if err != nil {
 		return nil, err
 	}
@@ -43,20 +45,18 @@ func (s Server) getDeploymentSpec() (*appsv1.Deployment, error) {
 		Name:  "CHE_SELF__SIGNED__CERT",
 		Value: "",
 	}
-	customPublicCertsVolumeSource := corev1.VolumeSource{}
-	customPublicCertsVolumeSource = corev1.VolumeSource{
-		ConfigMap: &corev1.ConfigMapVolumeSource{
-			LocalObjectReference: corev1.LocalObjectReference{
-				Name: deploy.CheAllCACertsConfigMapName,
+	customPublicCertsVolume := corev1.Volume{
+		Name: "che-public-certs",
+		VolumeSource: corev1.VolumeSource{
+			ConfigMap: &corev1.ConfigMapVolumeSource{
+				LocalObjectReference: corev1.LocalObjectReference{
+					Name: deploy.CheAllCACertsConfigMapName,
+				},
 			},
 		},
 	}
-	customPublicCertsVolume := corev1.Volume{
-		Name:         "che-public-certs",
-		VolumeSource: customPublicCertsVolumeSource,
-	}
 	customPublicCertsVolumeMount := corev1.VolumeMount{
-		Name:      "che-public-certs",
+		Name:      customPublicCertsVolume.Name,
 		MountPath: "/public-certs",
 	}
 	gitSelfSignedCertEnv := corev1.EnvVar{
@@ -67,7 +67,7 @@ func (s Server) getDeploymentSpec() (*appsv1.Deployment, error) {
 		Name:  "CHE_GIT_SELF__SIGNED__CERT__HOST",
 		Value: "",
 	}
-	if selfSignedCertUsed {
+	if selfSignedCASecretExists {
 		selfSignedCertEnv = corev1.EnvVar{
 			Name: "CHE_SELF__SIGNED__CERT",
 			ValueFrom: &corev1.EnvVarSource{
@@ -266,50 +266,41 @@ func (s Server) getDeploymentSpec() (*appsv1.Deployment, error) {
 		return nil, err
 	}
 
+	err = MountGitHubOAuthConfig(s.deployContext, deployment)
+	if err != nil {
+		return nil, err
+	}
+
+	err = MountGitLabOAuthConfig(s.deployContext, deployment)
+	if err != nil {
+		return nil, err
+	}
+
 	container := &deployment.Spec.Template.Spec.Containers[0]
-	cheMultiUser := deploy.GetCheMultiUser(s.deployContext.CheCluster)
-	if cheMultiUser == "true" {
-		chePostgresSecret := s.deployContext.CheCluster.Spec.Database.ChePostgresSecret
-		if len(chePostgresSecret) > 0 {
-			container.Env = append(container.Env,
-				corev1.EnvVar{
-					Name: "CHE_JDBC_USERNAME",
-					ValueFrom: &corev1.EnvVarSource{
-						SecretKeyRef: &corev1.SecretKeySelector{
-							Key: "user",
-							LocalObjectReference: corev1.LocalObjectReference{
-								Name: chePostgresSecret,
-							},
+	chePostgresSecret := s.deployContext.CheCluster.Spec.Database.ChePostgresSecret
+	if len(chePostgresSecret) > 0 {
+		container.Env = append(container.Env,
+			corev1.EnvVar{
+				Name: "CHE_JDBC_USERNAME",
+				ValueFrom: &corev1.EnvVarSource{
+					SecretKeyRef: &corev1.SecretKeySelector{
+						Key: "user",
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: chePostgresSecret,
 						},
 					},
-				}, corev1.EnvVar{
-					Name: "CHE_JDBC_PASSWORD",
-					ValueFrom: &corev1.EnvVarSource{
-						SecretKeyRef: &corev1.SecretKeySelector{
-							Key: "password",
-							LocalObjectReference: corev1.LocalObjectReference{
-								Name: chePostgresSecret,
-							},
-						},
-					},
-				})
-		}
-	} else {
-		deployment.Spec.Strategy.Type = appsv1.RecreateDeploymentStrategyType
-		deployment.Spec.Template.Spec.Volumes = []corev1.Volume{
-			{
-				Name: deploy.DefaultCheVolumeClaimName,
-				VolumeSource: corev1.VolumeSource{
-					PersistentVolumeClaim: &corev1.PersistentVolumeClaimVolumeSource{
-						ClaimName: deploy.DefaultCheVolumeClaimName,
-					},
 				},
-			}}
-		container.VolumeMounts = []corev1.VolumeMount{
-			{
-				MountPath: deploy.DefaultCheVolumeMountPath,
-				Name:      deploy.DefaultCheVolumeClaimName,
-			}}
+			}, corev1.EnvVar{
+				Name: "CHE_JDBC_PASSWORD",
+				ValueFrom: &corev1.EnvVarSource{
+					SecretKeyRef: &corev1.SecretKeySelector{
+						Key: "password",
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: chePostgresSecret,
+						},
+					},
+				},
+			})
 	}
 
 	// configure probes if debug isn't set
@@ -369,6 +360,24 @@ func (s Server) getDeploymentSpec() (*appsv1.Deployment, error) {
 		}
 	}
 
+	if deploy.IsComponentReadinessInitContainersConfigured(s.deployContext.CheCluster) {
+		if !s.deployContext.CheCluster.Spec.Database.ExternalDb {
+			waitForPostgresInitContainer, err := postgres.GetWaitForPostgresInitContainer(s.deployContext)
+			if err != nil {
+				return nil, err
+			}
+			deployment.Spec.Template.Spec.InitContainers = append(deployment.Spec.Template.Spec.InitContainers, *waitForPostgresInitContainer)
+		}
+
+		if !s.deployContext.CheCluster.Spec.Auth.ExternalIdentityProvider {
+			waitForKeycloakInitContainer, err := identity_provider.GetWaitForKeycloakInitContainer(s.deployContext)
+			if err != nil {
+				return nil, err
+			}
+			deployment.Spec.Template.Spec.InitContainers = append(deployment.Spec.Template.Spec.InitContainers, *waitForKeycloakInitContainer)
+		}
+	}
+
 	return deployment, nil
 }
 
@@ -395,7 +404,6 @@ func GetFullCheServerImageLink(checluster *orgv1.CheCluster) string {
 }
 
 func MountBitBucketOAuthConfig(deployContext *deploy.DeployContext, deployment *appsv1.Deployment) error {
-	// mount BitBucket configuration
 	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
 		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
 		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
@@ -408,36 +416,120 @@ func MountBitBucketOAuthConfig(deployContext *deploy.DeployContext, deployment *
 	} else if len(secrets) > 1 {
 		return errors.New("More than 1 BitBucket OAuth configuration secrets found")
 	} else if len(secrets) == 1 {
-		// mount secrets
-		container := &deployment.Spec.Template.Spec.Containers[0]
-		deployment.Spec.Template.Spec.Volumes = append(deployment.Spec.Template.Spec.Volumes,
-			corev1.Volume{
-				Name: secrets[0].Name,
-				VolumeSource: corev1.VolumeSource{
-					Secret: &corev1.SecretVolumeSource{
-						SecretName: secrets[0].Name,
-					},
-				},
-			})
-		container.VolumeMounts = append(container.VolumeMounts,
-			corev1.VolumeMount{
-				Name:      secrets[0].Name,
-				MountPath: deploy.BitBucketOAuthConfigMountPath,
-			})
-
-		// mount env
-		endpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
-		container.Env = append(container.Env, corev1.EnvVar{
-			Name:  "CHE_OAUTH1_BITBUCKET_CONSUMERKEYPATH",
-			Value: deploy.BitBucketOAuthConfigMountPath + "/" + deploy.BitBucketOAuthConfigConsumerKey,
-		}, corev1.EnvVar{
-			Name:  "CHE_OAUTH1_BITBUCKET_PRIVATEKEYPATH",
-			Value: deploy.BitBucketOAuthConfigMountPath + "/" + deploy.BitBucketOAuthConfigPrivateKey,
-		}, corev1.EnvVar{
-			Name:  "CHE_OAUTH1_BITBUCKET_ENDPOINT",
-			Value: endpoint,
+		mountSecret(deployment, &secrets[0], deploy.BitBucketOAuthConfigMountPath)
+		mountEnv(deployment, []corev1.EnvVar{
+			{
+				Name:  "CHE_OAUTH1_BITBUCKET_CONSUMERKEYPATH",
+				Value: deploy.BitBucketOAuthConfigMountPath + "/" + deploy.BitBucketOAuthConfigConsumerKeyFileName,
+			}, {
+				Name:  "CHE_OAUTH1_BITBUCKET_PRIVATEKEYPATH",
+				Value: deploy.BitBucketOAuthConfigMountPath + "/" + deploy.BitBucketOAuthConfigPrivateKeyFileName,
+			},
 		})
+
+		endpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
+		if endpoint != "" {
+			mountEnv(deployment, []corev1.EnvVar{{
+				Name:  "CHE_OAUTH1_BITBUCKET_ENDPOINT",
+				Value: endpoint,
+			}})
+		}
 	}
 
 	return nil
 }
+
+func MountGitHubOAuthConfig(deployContext *deploy.DeployContext, deployment *appsv1.Deployment) error {
+	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
+		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
+		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
+	}, map[string]string{
+		deploy.CheEclipseOrgOAuthScmServer: "github",
+	})
+
+	if err != nil {
+		return err
+	} else if len(secrets) > 1 {
+		return errors.New("More than 1 GitHub OAuth configuration secrets found")
+	} else if len(secrets) == 1 {
+		mountSecret(deployment, &secrets[0], deploy.GitHubOAuthConfigMountPath)
+		mountEnv(deployment, []corev1.EnvVar{
+			{
+				Name:  "CHE_OAUTH2_GITHUB_CLIENTID__FILEPATH",
+				Value: deploy.GitHubOAuthConfigMountPath + "/" + deploy.GitHubOAuthConfigClientIdFileName,
+			}, {
+				Name:  "CHE_OAUTH2_GITHUB_CLIENTSECRET__FILEPATH",
+				Value: deploy.GitHubOAuthConfigMountPath + "/" + deploy.GitHubOAuthConfigClientSecretFileName,
+			},
+		})
+
+		endpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
+		if endpoint != "" {
+			mountEnv(deployment, []corev1.EnvVar{{
+				Name:  "CHE_INTEGRATION_GITHUB_SERVER__ENDPOINTS",
+				Value: endpoint,
+			}})
+		}
+	}
+
+	return nil
+}
+
+func MountGitLabOAuthConfig(deployContext *deploy.DeployContext, deployment *appsv1.Deployment) error {
+	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
+		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
+		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
+	}, map[string]string{
+		deploy.CheEclipseOrgOAuthScmServer: "gitlab",
+	})
+
+	if err != nil {
+		return err
+	} else if len(secrets) > 1 {
+		return errors.New("More than 1 GitLab OAuth configuration secrets found")
+	} else if len(secrets) == 1 {
+		mountSecret(deployment, &secrets[0], deploy.GitLabOAuthConfigMountPath)
+		mountEnv(deployment, []corev1.EnvVar{
+			{
+				Name:  "CHE_OAUTH_GITLAB_CLIENTID__FILEPATH",
+				Value: deploy.GitLabOAuthConfigMountPath + "/" + deploy.GitLabOAuthConfigClientIdFileName,
+			}, {
+				Name:  "CHE_OAUTH_GITLAB_CLIENTSECRET__FILEPATH",
+				Value: deploy.GitLabOAuthConfigMountPath + "/" + deploy.GitLabOAuthConfigClientSecretFileName,
+			},
+		})
+
+		endpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
+		if endpoint != "" {
+			mountEnv(deployment, []corev1.EnvVar{{
+				Name:  "CHE_INTEGRATION_GITLAB_SERVER__ENDPOINTS",
+				Value: endpoint,
+			}})
+		}
+	}
+
+	return nil
+}
+
+func mountSecret(deployment *appsv1.Deployment, secret *corev1.Secret, mountPath string) {
+	container := &deployment.Spec.Template.Spec.Containers[0]
+	deployment.Spec.Template.Spec.Volumes = append(deployment.Spec.Template.Spec.Volumes,
+		corev1.Volume{
+			Name: secret.Name,
+			VolumeSource: corev1.VolumeSource{
+				Secret: &corev1.SecretVolumeSource{
+					SecretName: secret.Name,
+				},
+			},
+		})
+	container.VolumeMounts = append(container.VolumeMounts,
+		corev1.VolumeMount{
+			Name:      secret.Name,
+			MountPath: mountPath,
+		})
+}
+
+func mountEnv(deployment *appsv1.Deployment, envVar []corev1.EnvVar) {
+	container := &deployment.Spec.Template.Spec.Containers[0]
+	container.Env = append(container.Env, envVar...)
+}
