@@ -13,10 +13,9 @@ package server
 
 import (
 	"os"
-	"reflect"
 
 	"github.com/eclipse-che/che-operator/pkg/util"
-	"github.com/google/go-cmp/cmp"
+	"github.com/stretchr/testify/assert"
 
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 
@@ -26,8 +25,8 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/client-go/kubernetes/scheme"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
-	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 
 	"testing"
 )
@@ -82,7 +81,7 @@ func TestDeployment(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+			logf.SetLogger(zap.New(zap.WriteTo(os.Stdout), zap.UseDevMode(true)))
 			orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
 			testCase.initObjects = append(testCase.initObjects)
 			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
@@ -119,7 +118,6 @@ func TestMountBitBucketOAuthEnvVar(t *testing.T) {
 	type testCase struct {
 		name                       string
 		initObjects                []runtime.Object
-		cheCluster                 *orgv1.CheCluster
 		expectedConsumerKeyPathEnv corev1.EnvVar
 		expectedPrivateKeyPathEnv  corev1.EnvVar
 		expectedEndpointEnv        corev1.EnvVar
@@ -145,7 +143,7 @@ func TestMountBitBucketOAuthEnvVar(t *testing.T) {
 						},
 						Annotations: map[string]string{
 							"che.eclipse.org/oauth-scm-server":    "bitbucket",
-							"che.eclipse.org/scm-server-endpoint": "bitbucket_endpoint",
+							"che.eclipse.org/scm-server-endpoint": "endpoint",
 						},
 					},
 					Data: map[string][]byte{
@@ -154,11 +152,6 @@ func TestMountBitBucketOAuthEnvVar(t *testing.T) {
 					},
 				},
 			},
-			cheCluster: &orgv1.CheCluster{
-				ObjectMeta: metav1.ObjectMeta{
-					Namespace: "eclipse-che",
-				},
-			},
 			expectedConsumerKeyPathEnv: corev1.EnvVar{
 				Name:  "CHE_OAUTH1_BITBUCKET_CONSUMERKEYPATH",
 				Value: "/che-conf/oauth/bitbucket/consumer.key",
@@ -169,7 +162,7 @@ func TestMountBitBucketOAuthEnvVar(t *testing.T) {
 			},
 			expectedEndpointEnv: corev1.EnvVar{
 				Name:  "CHE_OAUTH1_BITBUCKET_ENDPOINT",
-				Value: "bitbucket_endpoint",
+				Value: "endpoint",
 			},
 			expectedVolume: corev1.Volume{
 				Name: "github-oauth-config",
@@ -188,51 +181,225 @@ func TestMountBitBucketOAuthEnvVar(t *testing.T) {
 
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
-			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
-			orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
-			testCase.initObjects = append(testCase.initObjects)
-			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
-
-			deployContext := &deploy.DeployContext{
-				CheCluster: testCase.cheCluster,
-				ClusterAPI: deploy.ClusterAPI{
-					Client: cli,
-					Scheme: scheme.Scheme,
-				},
-				Proxy: &deploy.Proxy{},
-			}
+			deployContext := deploy.GetTestDeployContext(nil, testCase.initObjects)
 
 			server := NewServer(deployContext)
 			deployment, err := server.getDeploymentSpec()
-			if err != nil {
-				t.Fatalf("Error creating deployment: %v", err)
-			}
+			assert.Nil(t, err, "Unexpected error occurred %v", err)
 
 			container := &deployment.Spec.Template.Spec.Containers[0]
+
 			env := util.FindEnv(container.Env, "CHE_OAUTH1_BITBUCKET_CONSUMERKEYPATH")
-			if !reflect.DeepEqual(testCase.expectedConsumerKeyPathEnv, *env) {
-				t.Errorf("Expected Env and Env returned from API server differ (-want, +got): %v", cmp.Diff(testCase.expectedConsumerKeyPathEnv, env))
-			}
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedConsumerKeyPathEnv, *env)
 
 			env = util.FindEnv(container.Env, "CHE_OAUTH1_BITBUCKET_PRIVATEKEYPATH")
-			if !reflect.DeepEqual(testCase.expectedPrivateKeyPathEnv, *env) {
-				t.Errorf("Expected Env and Env returned from API server differ (-want, +got): %v", cmp.Diff(testCase.expectedPrivateKeyPathEnv, env))
-			}
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedPrivateKeyPathEnv, *env)
 
 			env = util.FindEnv(container.Env, "CHE_OAUTH1_BITBUCKET_ENDPOINT")
-			if !reflect.DeepEqual(testCase.expectedEndpointEnv, *env) {
-				t.Errorf("Expected Env and Env returned from API server differ (-want, +got): %v", cmp.Diff(testCase.expectedEndpointEnv, env))
-			}
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedEndpointEnv, *env)
 
 			volume := util.FindVolume(deployment.Spec.Template.Spec.Volumes, "github-oauth-config")
-			if !reflect.DeepEqual(testCase.expectedVolume, volume) {
-				t.Errorf("Expected Volume and Volume returned from API server differ (-want, +got): %v", cmp.Diff(testCase.expectedVolume, volume))
-			}
+			assert.NotNil(t, volume)
+			assert.Equal(t, testCase.expectedVolume, volume)
 
 			volumeMount := util.FindVolumeMount(container.VolumeMounts, "github-oauth-config")
-			if !reflect.DeepEqual(testCase.expectedVolumeMount, volumeMount) {
-				t.Errorf("Expected VolumeMount and VolumeMount returned from API server differ (-want, +got): %v", cmp.Diff(testCase.expectedVolumeMount, volumeMount))
-			}
+			assert.NotNil(t, volumeMount)
+			assert.Equal(t, testCase.expectedVolumeMount, volumeMount)
+		})
+	}
+}
+
+func TestMountGitHubOAuthEnvVar(t *testing.T) {
+	type testCase struct {
+		name                     string
+		initObjects              []runtime.Object
+		expectedIdKeyPathEnv     corev1.EnvVar
+		expectedSecretKeyPathEnv corev1.EnvVar
+		expectedEndpointEnv      corev1.EnvVar
+		expectedVolume           corev1.Volume
+		expectedVolumeMount      corev1.VolumeMount
+	}
+
+	testCases := []testCase{
+		{
+			name: "Test",
+			initObjects: []runtime.Object{
+				&corev1.Secret{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "Secret",
+						APIVersion: "v1",
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      "github-oauth-config",
+						Namespace: "eclipse-che",
+						Labels: map[string]string{
+							"app.kubernetes.io/part-of":   "che.eclipse.org",
+							"app.kubernetes.io/component": "oauth-scm-configuration",
+						},
+						Annotations: map[string]string{
+							"che.eclipse.org/oauth-scm-server":    "github",
+							"che.eclipse.org/scm-server-endpoint": "endpoint",
+						},
+					},
+					Data: map[string][]byte{
+						"id":     []byte("some_id"),
+						"secret": []byte("some_secret"),
+					},
+				},
+			},
+			expectedIdKeyPathEnv: corev1.EnvVar{
+				Name:  "CHE_OAUTH2_GITHUB_CLIENTID__FILEPATH",
+				Value: "/che-conf/oauth/github/id",
+			},
+			expectedSecretKeyPathEnv: corev1.EnvVar{
+				Name:  "CHE_OAUTH2_GITHUB_CLIENTSECRET__FILEPATH",
+				Value: "/che-conf/oauth/github/secret",
+			},
+			expectedEndpointEnv: corev1.EnvVar{
+				Name:  "CHE_INTEGRATION_GITHUB_SERVER__ENDPOINTS",
+				Value: "endpoint",
+			},
+			expectedVolume: corev1.Volume{
+				Name: "github-oauth-config",
+				VolumeSource: corev1.VolumeSource{
+					Secret: &corev1.SecretVolumeSource{
+						SecretName: "github-oauth-config",
+					},
+				},
+			},
+			expectedVolumeMount: corev1.VolumeMount{
+				Name:      "github-oauth-config",
+				MountPath: "/che-conf/oauth/github",
+			},
+		},
+	}
+
+	for _, testCase := range testCases {
+		t.Run(testCase.name, func(t *testing.T) {
+			deployContext := deploy.GetTestDeployContext(nil, testCase.initObjects)
+
+			server := NewServer(deployContext)
+			deployment, err := server.getDeploymentSpec()
+			assert.Nil(t, err, "Unexpected error %v", err)
+
+			container := &deployment.Spec.Template.Spec.Containers[0]
+
+			env := util.FindEnv(container.Env, "CHE_OAUTH2_GITHUB_CLIENTID__FILEPATH")
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedIdKeyPathEnv, *env)
+
+			env = util.FindEnv(container.Env, "CHE_OAUTH2_GITHUB_CLIENTSECRET__FILEPATH")
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedSecretKeyPathEnv, *env)
+
+			volume := util.FindVolume(deployment.Spec.Template.Spec.Volumes, "github-oauth-config")
+			assert.NotNil(t, volume)
+			assert.Equal(t, testCase.expectedVolume, volume)
+
+			volumeMount := util.FindVolumeMount(container.VolumeMounts, "github-oauth-config")
+			assert.NotNil(t, volumeMount)
+			assert.Equal(t, testCase.expectedVolumeMount, volumeMount)
+		})
+	}
+}
+
+func TestMountGitLabOAuthEnvVar(t *testing.T) {
+	type testCase struct {
+		name                     string
+		initObjects              []runtime.Object
+		expectedIdKeyPathEnv     corev1.EnvVar
+		expectedSecretKeyPathEnv corev1.EnvVar
+		expectedEndpointEnv      corev1.EnvVar
+		expectedVolume           corev1.Volume
+		expectedVolumeMount      corev1.VolumeMount
+	}
+
+	testCases := []testCase{
+		{
+			name: "Test",
+			initObjects: []runtime.Object{
+				&corev1.Secret{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "Secret",
+						APIVersion: "v1",
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      "gitlab-oauth-config",
+						Namespace: "eclipse-che",
+						Labels: map[string]string{
+							"app.kubernetes.io/part-of":   "che.eclipse.org",
+							"app.kubernetes.io/component": "oauth-scm-configuration",
+						},
+						Annotations: map[string]string{
+							"che.eclipse.org/oauth-scm-server":    "gitlab",
+							"che.eclipse.org/scm-server-endpoint": "endpoint",
+						},
+					},
+					Data: map[string][]byte{
+						"id":     []byte("some_id"),
+						"secret": []byte("some_secret"),
+					},
+				},
+			},
+			expectedIdKeyPathEnv: corev1.EnvVar{
+				Name:  "CHE_OAUTH_GITLAB_CLIENTID__FILEPATH",
+				Value: "/che-conf/oauth/gitlab/id",
+			},
+			expectedSecretKeyPathEnv: corev1.EnvVar{
+				Name:  "CHE_OAUTH_GITLAB_CLIENTSECRET__FILEPATH",
+				Value: "/che-conf/oauth/gitlab/secret",
+			},
+			expectedEndpointEnv: corev1.EnvVar{
+				Name:  "CHE_INTEGRATION_GITLAB_SERVER__ENDPOINTS",
+				Value: "endpoint",
+			},
+			expectedVolume: corev1.Volume{
+				Name: "gitlab-oauth-config",
+				VolumeSource: corev1.VolumeSource{
+					Secret: &corev1.SecretVolumeSource{
+						SecretName: "gitlab-oauth-config",
+					},
+				},
+			},
+			expectedVolumeMount: corev1.VolumeMount{
+				Name:      "gitlab-oauth-config",
+				MountPath: "/che-conf/oauth/gitlab",
+			},
+		},
+	}
+
+	for _, testCase := range testCases {
+		t.Run(testCase.name, func(t *testing.T) {
+			deployContext := deploy.GetTestDeployContext(nil, testCase.initObjects)
+
+			server := NewServer(deployContext)
+			deployment, err := server.getDeploymentSpec()
+			assert.Nil(t, err, "Unexpected error %v", err)
+
+			container := &deployment.Spec.Template.Spec.Containers[0]
+
+			env := util.FindEnv(container.Env, "CHE_OAUTH_GITLAB_CLIENTID__FILEPATH")
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedIdKeyPathEnv, *env)
+
+			env = util.FindEnv(container.Env, "CHE_OAUTH_GITLAB_CLIENTSECRET__FILEPATH")
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedSecretKeyPathEnv, *env)
+
+			env = util.FindEnv(container.Env, "CHE_INTEGRATION_GITLAB_SERVER__ENDPOINTS")
+			assert.NotNil(t, env)
+			assert.Equal(t, testCase.expectedEndpointEnv, *env)
+
+			volume := util.FindVolume(deployment.Spec.Template.Spec.Volumes, "gitlab-oauth-config")
+			assert.NotNil(t, volume)
+			assert.Equal(t, testCase.expectedVolume, volume)
+
+			volumeMount := util.FindVolumeMount(container.VolumeMounts, "gitlab-oauth-config")
+			assert.NotNil(t, volumeMount)
+			assert.Equal(t, testCase.expectedVolumeMount, volumeMount)
 		})
 	}
 }
