@@ -47,8 +47,6 @@ const (
 	CheTLSSelfSignedCertificateSecretName = "self-signed-certificate"
 	DefaultCheTLSSecretName               = "che-tls"
 
-	// CheCACertsConfigMapLabelKey is the label key which marks config map with additional CA certificates
-	CheCACertsConfigMapLabelKey = "app.kubernetes.io/component"
 	// CheCACertsConfigMapLabelKey is the label value which marks config map with additional CA certificates
 	CheCACertsConfigMapLabelValue = "ca-bundle"
 	// CheAllCACertsConfigMapName is the name of config map which contains all additional trusted by Che TLS CA certificates
@@ -63,23 +61,34 @@ const (
 	labelCommaSign = "."
 )
 
+// IsSelfSignedCASecretExists checks if CheTLSSelfSignedCertificateSecretName exists so depending components can mount it
+func IsSelfSignedCASecretExists(deployContext *DeployContext) (bool, error) {
+	cheTLSSelfSignedCertificateSecret := &corev1.Secret{}
+	err := deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: CheTLSSelfSignedCertificateSecretName}, cheTLSSelfSignedCertificateSecret)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return false, nil
+		}
+		return false, err
+	}
+	return true, nil
+}
+
 // IsSelfSignedCertificateUsed detects whether endpoints are/should be secured by self-signed certificate.
 func IsSelfSignedCertificateUsed(deployContext *DeployContext) (bool, error) {
 	if util.IsTestMode() {
 		return true, nil
 	}
 
-	cheTLSSelfSignedCertificateSecret := &corev1.Secret{}
-	err := deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: CheTLSSelfSignedCertificateSecretName}, cheTLSSelfSignedCertificateSecret)
-	if err == nil {
-		// "self signed-certificate" secret found
-		return true, nil
-	}
-	if !errors.IsNotFound(err) {
-		// Failed to get secret, return error to restart reconcile loop.
+	cheCASecretExist, err := IsSelfSignedCASecretExists(deployContext)
+	if err != nil {
 		return false, err
 	}
 
+	if cheCASecretExist {
+		return true, nil
+	}
+
 	if !util.IsOpenShift {
 		// Handle custom tls secret for Che ingresses
 		cheTLSSecretName := deployContext.CheCluster.Spec.K8s.TlsSecretName
@@ -580,10 +589,11 @@ func SyncAdditionalCACertsConfigMapToCluster(deployContext *DeployContext) (bool
 func GetCACertsConfigMaps(deployContext *DeployContext) ([]corev1.ConfigMap, error) {
 	CACertsConfigMapList := &corev1.ConfigMapList{}
 
-	caBundleLabelSelectorRequirement, _ := labels.NewRequirement(CheCACertsConfigMapLabelKey, selection.Equals, []string{CheCACertsConfigMapLabelValue})
+	caBundleLabelSelectorRequirement, _ := labels.NewRequirement(KubernetesComponentLabelKey, selection.Equals, []string{CheCACertsConfigMapLabelValue})
 	cheComponetLabelSelectorRequirement, _ := labels.NewRequirement(KubernetesPartOfLabelKey, selection.Equals, []string{CheEclipseOrg})
 	listOptions := &client.ListOptions{
 		LabelSelector: labels.NewSelector().Add(*cheComponetLabelSelectorRequirement).Add(*caBundleLabelSelectorRequirement),
+		Namespace:     deployContext.CheCluster.GetNamespace(),
 	}
 	if err := deployContext.ClusterAPI.Client.List(context.TODO(), CACertsConfigMapList, listOptions); err != nil {
 		return nil, err
