@@ -561,32 +561,6 @@ func ComputeHash256(data []byte) string {
 	return base64.URLEncoding.EncodeToString(hasher.Sum(nil))
 }
 
-func ReloadCheCluster(client client.Client, cheCluster *orgv1.CheCluster) error {
-	return client.Get(
-		context.TODO(),
-		types.NamespacedName{Name: cheCluster.Name, Namespace: cheCluster.Namespace},
-		cheCluster)
-}
-
-func FindCheCRinNamespace(client client.Client, namespace string) (*orgv1.CheCluster, int, error) {
-	cheClusters := &orgv1.CheClusterList{}
-	if err := client.List(context.TODO(), cheClusters); err != nil {
-		return nil, -1, err
-	}
-
-	if len(cheClusters.Items) != 1 {
-		return nil, len(cheClusters.Items), fmt.Errorf("expected an instance of CheCluster, but got %d instances", len(cheClusters.Items))
-	}
-
-	cheCR := &orgv1.CheCluster{}
-	namespacedName := types.NamespacedName{Namespace: namespace, Name: cheClusters.Items[0].GetName()}
-	err := client.Get(context.TODO(), namespacedName, cheCR)
-	if err != nil {
-		return nil, -1, err
-	}
-	return cheCR, 1, nil
-}
-
 func UpdateBackupServerConfiguration(client client.Client, backupServerConfig *orgv1.CheBackupServerConfiguration) error {
 	err := client.Update(context.TODO(), backupServerConfig)
 	if err != nil {
@@ -605,6 +579,10 @@ func UpdateBackupServerConfigurationStatus(client client.Client, backupServerCon
 	return nil
 }
 
+func IsCheMultiUser(cheCluster *orgv1.CheCluster) bool {
+	return cheCluster.Spec.Server.CustomCheProperties == nil || cheCluster.Spec.Server.CustomCheProperties["CHE_MULTIUSER"] != "false"
+}
+
 // ClearMetadata removes extra fields from given metadata.
 // It is required to remove ResourceVersion in order to be able to apply the yaml again.
 func ClearMetadata(objectMeta *metav1.ObjectMeta) {
